import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.ComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

/**
 * Music Management Client 1.0
 * 
 * MODULE: AComboBox
 * 
 * AUTHOR: Andy Maskin
 * 
 * PURPOSE: This is an extension of JComboBox that acts as an auto-complete combo box
 * As the user types, the combo box attempts to fill out the rest of what the user is trying to type
 * it makes dealing with big lists much more tolerable.
 *
 */

public class AComboBox extends JComboBox	implements JComboBox.KeySelectionManager
{
	private static final long serialVersionUID = 1L;
	private String searchFor;
	private long lap;
	public static String sLastValidChoice = "";
	
	public class CBDocument extends PlainDocument
	{
		private static final long serialVersionUID = 1L;

		public void insertString(int offset, String str, AttributeSet a) throws BadLocationException
		{
			if (str==null) return;
			super.insertString(offset, str, a);
			if(!isPopupVisible() && str.length() != 0) fireActionEvent();
		}
	}
	
	//NULL Constructor
	public AComboBox() {
		super();
		lap = new java.util.Date().getTime();
		setKeySelectionManager(this);
		JTextField tf;
		if(getEditor() != null)
		{
			tf = (JTextField)getEditor().getEditorComponent();
			if(tf != null)
			{
				tf.setDocument(new CBDocument());
				addActionListener(new ActionListener()
				{
					public void actionPerformed(ActionEvent evt)
					{
						boolean bNoMatch;
						JTextField tf = (JTextField)getEditor().getEditorComponent();
						String text = tf.getText();
						ComboBoxModel aModel = getModel();
						String current;
						bNoMatch = true;
						for(int i = 0; i < aModel.getSize(); i++)
						{
							current = aModel.getElementAt(i).toString();
							if(current.toLowerCase().startsWith(text.toLowerCase()))
							{
								tf.setText(current);
								tf.setSelectionStart(text.length());
								tf.setSelectionEnd(current.length());
								sLastValidChoice = current;
								bNoMatch = false;
								break;
							} 
						}
						if (bNoMatch && tf.getText().length() > 0) {
							//tf.setText(newText);
							tf.setText(sLastValidChoice);
						}
					}
				});
			}
		}
	} //end null constructor
	
	public int selectionForKey(char aKey, ComboBoxModel aModel)
	{
		long now = new java.util.Date().getTime();
		if (searchFor!=null && aKey==KeyEvent.VK_BACK_SPACE &&	searchFor.length()>0)
		{
			searchFor = searchFor.substring(0, searchFor.length() -1);
		}
		else
		{
			//	System.out.println(lap);
			if(lap + 1000 < now)
				searchFor = "" + aKey;
			else
				searchFor = searchFor + aKey;
		}
		lap = now;
		String current;
		for(int i = 0; i < aModel.getSize(); i++)
		{
			current = aModel.getElementAt(i).toString().toLowerCase();
			if (current.toLowerCase().startsWith(searchFor.toLowerCase())) return i;
		}
		return -1;
	}
	public void fireActionEvent()
	{
		super.fireActionEvent();
	}
	
	public int getAnIndex(String sInput) {
		int iReturnVal = 0;

		ComboBoxModel aModel = getModel();
		
		String sCurrent;
		for(int i = 0; i < aModel.getSize(); i++)
		{
			sCurrent = aModel.getElementAt(i).toString();
			if (sCurrent.equals(sInput)) {iReturnVal = i;}
		}
		
		return iReturnVal;
	}
	
}