import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;

/**
 * Music Management Client 1.1
 * 
 * MODULE: AllTagsPanel
 * 
 * AUTHOR: Andy Maskin
 * 
 * PURPOSE: This panel is where users can query all the files by a variety of different 
 * criteria and edit the tag information. You can do any one of these functions on other tabs, but
 * this tab allows you do do artist-independent queries
 *
 */

public class AllTagsPanel extends JPanel implements TableModelListener, ActionListener {

	private static final long serialVersionUID = 1L;
	public static Connection coDB;
	
	// GRAPHIC ELEMENTS
	JLabel lblResults = new JLabel("");
	JButton btnQuery = new JButton("Query");
	JScrollPane spQuery = new JScrollPane();
	JTable tbQuery = new JTable(1,16);
    AndyGrid tbResults = new AndyGrid(0,0);
    JScrollPane spResults;
    
    String sWhereClause;
    
    GridBagConstraints cStatus = new GridBagConstraints();
    GridBagConstraints cLabels = new GridBagConstraints();
    GridBagConstraints cFields = new GridBagConstraints();
	
    {
    		//PASSED-IN VALUES
    		coDB = MMCObject.coDB;
    		cStatus = MMCObject.cStatus;
    		cLabels = MMCObject.cLabels;
    		cFields = MMCObject.cFields;
    	
    		//LAYOUT STUFF
		spResults = new JScrollPane(tbResults);
		tbResults.setPreferredScrollableViewportSize(new Dimension(700, 350));
		spResults.setMinimumSize(new Dimension(700,350));
		
		tbQuery.setPreferredScrollableViewportSize(new Dimension(700, 25));
		tbQuery.setMinimumSize(new Dimension(700,25));
		tbQuery.setAutoResizeMode(0);
		tbQuery.setGridColor(Color.gray);
		TableColumn column = new TableColumn();
		column = tbQuery.getColumnModel().getColumn(0);		column.setHeaderValue("Filename");
		column = tbQuery.getColumnModel().getColumn(1); 		column.setHeaderValue("Artist");
		column = tbQuery.getColumnModel().getColumn(2); 		column.setHeaderValue("NewArtist");
		column = tbQuery.getColumnModel().getColumn(3); 		column.setHeaderValue("Title");
		column = tbQuery.getColumnModel().getColumn(4); 		column.setHeaderValue("NewTitle");
		column = tbQuery.getColumnModel().getColumn(5); 		column.setHeaderValue("Album");
		column = tbQuery.getColumnModel().getColumn(6); 		column.setHeaderValue("NewAlbum");
		column = tbQuery.getColumnModel().getColumn(7); 		column.setHeaderValue("Track");
		column = tbQuery.getColumnModel().getColumn(8); 		column.setHeaderValue("NewTrack");
		column = tbQuery.getColumnModel().getColumn(9); 		column.setHeaderValue("Year");
		column = tbQuery.getColumnModel().getColumn(10); 		column.setHeaderValue("NewYear");
		column = tbQuery.getColumnModel().getColumn(11); 		column.setHeaderValue("Genre");
		column = tbQuery.getColumnModel().getColumn(12); 		column.setHeaderValue("NewGenre");
		column = tbQuery.getColumnModel().getColumn(13); 		column.setHeaderValue("Comment");
		column = tbQuery.getColumnModel().getColumn(14); 		column.setHeaderValue("NewComment");
		column = tbQuery.getColumnModel().getColumn(15); 		column.setHeaderValue("OrigArtist");
    	
		spQuery = new JScrollPane(tbQuery);
		spQuery.setMinimumSize(new Dimension(700,50));
		spQuery.setSize(new Dimension(700,50));
		spQuery.setPreferredSize(new Dimension(700,50));
		
		btnQuery.addActionListener(this);
		
		this.setLayout(new GridBagLayout());
		this.add(btnQuery,cLabels);
	    	this.add(lblResults,cFields);
		this.add(spQuery,cStatus);
	    this.add(spResults,cStatus);
    }
    
    public void actionPerformed(ActionEvent arg0) {
		//System.out.println(arg0.toString());
		//System.out.println(arg0.getModifiers());
    	if (arg0.getActionCommand().equals("Query")) {
    		vBuildWhereClause();
	    	    Statement stSQL;
			ResultSet rsResults;
			String sSQL;
			
			// If no criteria where entered, prevents avalanche of data
			if (sWhereClause.equals("Filename = Filename ")) {sWhereClause = "Filename <> Filename ";}
			
			sSQL = "SELECT Filename, Filename as File, ";
			sSQL = sSQL + "Artist, NewArtist, Title, NewTitle, Album, NewAlbum, Track, NewTrack, ";
			sSQL = sSQL + "Year, NewYear, Genre, NewGenre, Comment, NewComment, OrigArtist ";
			sSQL = sSQL + "from Music WHERE " + sWhereClause + " ORDER BY Filename";
			
			System.out.println(sSQL);
			
			try {
				Vector columnNames = new Vector();
				Vector data = new Vector();
				stSQL = coDB.createStatement();
				rsResults = stSQL.executeQuery(sSQL);
	
				ResultSetMetaData mdResults = rsResults.getMetaData();
				int iColumnCount = mdResults.getColumnCount();
				
				if (rsResults.first()) {System.out.println("Records Returned");}
				
				//get ColumnNames
				for(int i=1; i <= iColumnCount ;i++){
					columnNames.addElement( mdResults.getColumnName(i) );
				}
				
				//get data
				boolean bContinue = rsResults.first();
				while (bContinue)
	            {
	                Vector row = new Vector(iColumnCount);
	                for (int i = 1; i <= iColumnCount; i++)
	                {
	                    row.addElement( rsResults.getObject(i) );
	                }
	                data.addElement( row );
	                bContinue = rsResults.next();
	            }
				
		        rsResults.close();
	            stSQL.close();
	            AndyGrid tbTemp = new AndyGrid(data, columnNames); 
	            setResultsTable(tbTemp);
				
			} // end try
			catch (SQLException e) {
				e.printStackTrace();
				System.out.println("SQL Error: " + e.getMessage());
				Vector columnNames = new Vector();
				Vector data = new Vector();
				Vector row = new Vector(1);
				columnNames.addElement("X");
				columnNames.addElement("SQL ERROR");
				row.addElement("!");
				row.addElement(e.getMessage());
				data.addElement(row);
				AndyGrid tbTemp = new AndyGrid(data, columnNames); 
	            	this.setResultsTable(tbTemp);
			} //end exception
		} // end if edited
	}

    private void vBuildWhereClause() {
    		sWhereClause = "Filename = Filename ";
    		String sInputString;
    		
    		if (tbQuery.getValueAt(0,0) != null) {
        		sInputString = sCleanString(tbQuery.getValueAt(0,0).toString());
    		} else {sInputString = "";}
    		if (sInputString.equals("") == false) {sWhereClause = sWhereClause + "AND Filename Like '%" + sInputString + "%' COLLATE latin1_general_cs ";}

    		if (tbQuery.getValueAt(0,1) != null) {
        		sInputString = sCleanString(tbQuery.getValueAt(0,1).toString());
    		} else {sInputString = "";}
    		if (sInputString.equals("") == false) {sWhereClause = sWhereClause + "AND Artist Like '%" + sInputString + "%' COLLATE latin1_general_cs ";}
    		
    		if (tbQuery.getValueAt(0,2) != null) {
        		sInputString = sCleanString(tbQuery.getValueAt(0,2).toString());
    		} else {sInputString = "";}
    		if (sInputString.equals("") == false) {sWhereClause = sWhereClause + "AND NewArtist Like '%" + sInputString + "%' COLLATE latin1_general_cs ";}

    		if (tbQuery.getValueAt(0,3) != null) {
        		sInputString = sCleanString(tbQuery.getValueAt(0,3).toString());
    		} else {sInputString = "";}
    		if (sInputString.equals("") == false) {sWhereClause = sWhereClause + "AND Title Like '%" + sInputString + "%' COLLATE latin1_general_cs ";}

    		if (tbQuery.getValueAt(0,4) != null) {
        		sInputString = sCleanString(tbQuery.getValueAt(0,4).toString());
    		} else {sInputString = "";}
    		if (sInputString.equals("") == false) {sWhereClause = sWhereClause + "AND NewTitle Like '%" + sInputString + "%' COLLATE latin1_general_cs ";}

    		if (tbQuery.getValueAt(0,5) != null) {
        		sInputString = sCleanString(tbQuery.getValueAt(0,5).toString());
    		} else {sInputString = "";}
    		if (sInputString.equals("") == false) {sWhereClause = sWhereClause + "AND Album Like '%" + sInputString + "%' COLLATE latin1_general_cs ";}

    		if (tbQuery.getValueAt(0,6) != null) {
        		sInputString = sCleanString(tbQuery.getValueAt(0,6).toString());
    		} else {sInputString = "";}
    		if (sInputString.equals("") == false) {sWhereClause = sWhereClause + "AND NewAlbum Like '%" + sInputString + "%' COLLATE latin1_general_cs ";}

    		if (tbQuery.getValueAt(0,7) != null) {
        		sInputString = sCleanString(tbQuery.getValueAt(0,7).toString());
    		} else {sInputString = "";}
    		if (sInputString.equals("") == false) {sWhereClause = sWhereClause + "AND Track Like '%" + sInputString + "%' COLLATE latin1_general_cs ";}

    		if (tbQuery.getValueAt(0,8) != null) {
        		sInputString = sCleanString(tbQuery.getValueAt(0,8).toString());
    		} else {sInputString = "";}
    		if (sInputString.equals("") == false) {sWhereClause = sWhereClause + "AND NewTrack Like '%" + sInputString + "%' COLLATE latin1_general_cs ";}

    		if (tbQuery.getValueAt(0,9) != null) {
        		sInputString = sCleanString(tbQuery.getValueAt(0,9).toString());
    		} else {sInputString = "";}
    		if (sInputString.equals("") == false) {sWhereClause = sWhereClause + "AND Year Like '%" + sInputString + "%' COLLATE latin1_general_cs ";}

    		if (tbQuery.getValueAt(0,10) != null) {
        		sInputString = sCleanString(tbQuery.getValueAt(0,10).toString());
    		} else {sInputString = "";}
    		if (sInputString.equals("") == false) {sWhereClause = sWhereClause + "AND NewYear Like '%" + sInputString + "%' COLLATE latin1_general_cs ";}

    		if (tbQuery.getValueAt(0,11) != null) {
        		sInputString = sCleanString(tbQuery.getValueAt(0,11).toString());
    		} else {sInputString = "";}
    		if (sInputString.equals("") == false) {sWhereClause = sWhereClause + "AND Genre Like '%" + sInputString + "%' COLLATE latin1_general_cs ";}

    		if (tbQuery.getValueAt(0,12) != null) {
        		sInputString = sCleanString(tbQuery.getValueAt(0,12).toString());
    		} else {sInputString = "";}
    		if (sInputString.equals("") == false) {sWhereClause = sWhereClause + "AND NewGenre Like '%" + sInputString + "%' COLLATE latin1_general_cs ";}

    		if (tbQuery.getValueAt(0,13) != null) {
        		sInputString = sCleanString(tbQuery.getValueAt(0,13).toString());
    		} else {sInputString = "";}
    		if (sInputString.equals("") == false) {sWhereClause = sWhereClause + "AND Comment Like '%" + sInputString + "%' COLLATE latin1_general_cs ";}

    		if (tbQuery.getValueAt(0,14) != null) {
        		sInputString = sCleanString(tbQuery.getValueAt(0,14).toString());
    		} else {sInputString = "";}
    		if (sInputString.equals("") == false) {sWhereClause = sWhereClause + "AND NewComment Like '%" + sInputString + "%' COLLATE latin1_general_cs ";}

    		if (tbQuery.getValueAt(0,15) != null) {
        		sInputString = sCleanString(tbQuery.getValueAt(0,15).toString());
    		} else {sInputString = "";}
    		if (sInputString.equals("") == false) {sWhereClause = sWhereClause + "AND OrigArtist Like '%" + sInputString + "%' COLLATE latin1_general_cs ";}

    		System.out.println(sWhereClause);
	}
    
    public String sCleanString(String _sInput) {
    		String sReturnVal = "";
    		sReturnVal = _sInput.replaceAll("\'","\\\\" + "\'");
    		sReturnVal = sReturnVal.replaceAll("\\*","%");
    		return sReturnVal;
    }

	private void setResultsTable(AndyGrid _tbResults) {
		tbResults = _tbResults;
		System.out.println((tbResults.getColumnCount()) + " columns");
		System.out.println((tbResults.getRowCount()) + " rows");
	    this.remove(spResults);
	    tbResults.formatGrid();
	    tbResults.getModel().addTableModelListener(this);
	    spResults.add(tbResults);
		spResults = new JScrollPane(tbResults);
	    spResults.setMinimumSize(new Dimension(700,350));
		spResults.setSize(new Dimension(700,350));
		spResults.setPreferredSize(new Dimension(700,350));
	    //spResults.setHorizontalScrollBarPolicy(32);
	    //spResults.setVerticalScrollBarPolicy(22);
		lblResults.setText(tbResults.getRowCount() + " Hits");
	    this.add(spResults,cStatus);
	    this.repaint();
	    this.revalidate();
	    spResults.requestFocus();
	    tbResults.requestFocus();
	}

	public void tableChanged(TableModelEvent e) {
		int row = e.getFirstRow();
        	int column = e.getColumn();
        	if ( column != 0) {
	        	TableModel model = (TableModel)e.getSource();
	        tbResults.captureUpdate(row, column, (String) model.getValueAt(row, column));
        } //cannot edit key column
    }
    
} //END CLASS
