import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 * Music Management Client 1.0
 * 
 * MODULE: AnalysisPanel
 * 
 * AUTHOR: Andy Maskin
 * 
 * PURPOSE: This panel is where any number of Analysis queries are visible
 *
 */

public class AnalysisPanel extends JPanel implements TableModelListener, ActionListener {

	private static final long serialVersionUID = 1L;
	public static Connection coDB;
	
	// GRAPHIC ELEMENTS
	public static JLabel lblName = new JLabel("Analysis: ");
	public static JComboBox cmbPossibleAnalysis = new JComboBox ();
    public static AndyGrid tbResults = new AndyGrid(0,0);
    public static JScrollPane spResults;
    
    GridBagConstraints cStatus = new GridBagConstraints();
    GridBagConstraints cLabels = new GridBagConstraints();
    GridBagConstraints cFields = new GridBagConstraints();
	
    {
    		//PASSED-IN VALUES
    		coDB = MMCObject.coDB;
    		cmbPossibleAnalysis = MMCObject.cmbPossibleAnalysis;
    		cStatus = MMCObject.cStatus;
    		cLabels = MMCObject.cLabels;
    		cFields = MMCObject.cFields;
    	
    		//LAYOUT STUFF
		spResults = new JScrollPane(tbResults);
		tbResults.setPreferredScrollableViewportSize(new Dimension(700, 500));
		spResults.setMinimumSize(new Dimension(700,500));
		

		cmbPossibleAnalysis.setEditable(false);
		cmbPossibleAnalysis.addActionListener(this);
    	
		this.setLayout(new GridBagLayout());
		this.add(lblName,cLabels);
	    	this.add(cmbPossibleAnalysis,cFields);
	    this.add(spResults,cStatus);
	    
    }
    
    public void actionPerformed(ActionEvent arg0) {
		if (arg0.getActionCommand().equals("comboBoxEdited") ||
				(arg0.getActionCommand().equals("comboBoxChanged") && arg0.getModifiers()==16)) {
	    	    Statement stSQL;
			ResultSet rsResults;
			String sSQL = "SELECT Artist as TheKey, Artist, Title From Music WHERE Filename = \"XXX\"";
			String sSelected;
			
			sSelected = (String) cmbPossibleAnalysis.getSelectedItem();
			
			if (sSelected.equals("Possible Duplicates")) {			
				sSQL = "SELECT 'key' as TheKey, a.Artist, a.Title, RIGHT(LEFT(b.FilenameWithPath,13),2) as ArchiveFolder, "; 
				sSQL = sSQL + "RIGHT(b.FilenameWithPath,LENGTH(b.FilenameWithPath)-14) as Filename From  ";
				sSQL = sSQL + "(SELECT c.Artist, c.Title, Count(c.Filename) as count from Music c group by c.Artist, c.Title) a, Music b ";
				sSQL = sSQL + "Where a.count  > 1 and a.Artist = b.Artist and a.Title = b.Title ";
				sSQL = sSQL + "ORDER BY a.Artist, a.Title, b.FilenameWithPath;";	
			} //end Possible Duplicates
			
			if (sSelected.equals("File Count By Artist")) {			
				sSQL = "SELECT 'key' as TheKey, ArtistFromFilename as A_Artist, Count(Filename) As Count From Music Group By ArtistFromFilename ";
				sSQL = sSQL + " Order By Count Desc, ArtistFromFilename";	
			} //end File Count By Artist
			
			if (sSelected.equals("Top Three Mix Genres Per User")) {			
				sSQL = "SELECT '---' as NoKey, v.User, v.Genre, v.FileCount as Count FROM ";
				sSQL = sSQL + "(SELECT w.User, w.Genre, w.FileCount, w.One, w.Two,  (SELECT ";
				sSQL = sSQL + "Max(y.FileCount) from (SELECT a.User, a.Genre, COUNT(a.Filename) as  ";
				sSQL = sSQL + "FileCount FROM (SELECT p.User, pe.Filename, m.Genre from Playlist p,  ";
				sSQL = sSQL + "PlaylistEntry pe, Music m WHERE p.Type = 'MIX' AND p.ROW_ID = pe.Playlist_ID and pe.Filename "; 
				sSQL = sSQL + "= m.Filename) a GROUP BY a.User, a.Genre) y where y.User = w.User and  ";
				sSQL = sSQL + "y.FileCount < w.Two) as Three FROM (SELECT x.User, x.Genre, x.FileCount,  ";
				sSQL = sSQL + "x.One, (SELECT Max(y.FileCount) from (SELECT a.User, a.Genre,  ";
				sSQL = sSQL + "COUNT(a.Filename) as FileCount FROM (SELECT p.User, pe.Filename, m.Genre "; 
				sSQL = sSQL + "from Playlist p, PlaylistEntry pe, Music m WHERE p.Type = 'MIX' AND p.ROW_ID = pe.Playlist_ID  ";
				sSQL = sSQL + "and pe.Filename = m.Filename) a GROUP BY a.User, a.Genre) y where y.User =  ";
				sSQL = sSQL + "x.User and y.FileCount < x.One) as Two FROM (SELECT z.User, z.Genre,  ";
				sSQL = sSQL + "z.FileCount, (SELECT Max(y.FileCount) from (SELECT a.User, a.Genre,  ";
				sSQL = sSQL + "COUNT(a.Filename) as FileCount FROM (SELECT p.User, pe.Filename, m.Genre  ";
				sSQL = sSQL + "from Playlist p, PlaylistEntry pe, Music m WHERE p.Type = 'MIX' AND p.ROW_ID = pe.Playlist_ID  ";
				sSQL = sSQL + "and pe.Filename = m.Filename) a GROUP BY a.User, a.Genre) y where y.User =  ";
				sSQL = sSQL + "z.User) as One From (SELECT a.User, a.Genre, COUNT(a.Filename) as  ";
				sSQL = sSQL + "FileCount FROM (SELECT p.User, pe.Filename, m.Genre from Playlist p,  ";
				sSQL = sSQL + "PlaylistEntry pe, Music m WHERE p.Type = 'MIX' AND p.ROW_ID = pe.Playlist_ID and pe.Filename  ";
				sSQL = sSQL + "= m.Filename) a GROUP BY a.User, a.Genre) z) x) w) v ";
				sSQL = sSQL + "where v.FileCount = v.One OR v.FileCount = v.Two OR v.FileCount = v.Three ";
				sSQL = sSQL + "ORDER BY v.User, v.FileCount DESC ";	
			} //end Top Three User Genre
			
			if (sSelected.equals("Top Three Mix Artists Per User")) {			
				sSQL = "SELECT '---' as NoKey, v.User, v.ArtistFromFilename as Artist, v.FileCount as Count FROM ";
				sSQL = sSQL + "(SELECT w.User, w.ArtistFromFilename, w.FileCount, w.One, w.Two,  (SELECT ";
				sSQL = sSQL + "Max(y.FileCount) from (SELECT a.User, a.ArtistFromFilename, COUNT(a.Filename) as  ";
				sSQL = sSQL + "FileCount FROM (SELECT p.User, pe.Filename, m.ArtistFromFilename from Playlist p,  ";
				sSQL = sSQL + "PlaylistEntry pe, Music m WHERE p.Type = 'MIX' AND p.ROW_ID = pe.Playlist_ID and pe.Filename "; 
				sSQL = sSQL + "= m.Filename) a GROUP BY a.User, a.ArtistFromFilename) y where y.User = w.User and  ";
				sSQL = sSQL + "y.FileCount < w.Two) as Three FROM (SELECT x.User, x.ArtistFromFilename, x.FileCount,  ";
				sSQL = sSQL + "x.One, (SELECT Max(y.FileCount) from (SELECT a.User, a.ArtistFromFilename,  ";
				sSQL = sSQL + "COUNT(a.Filename) as FileCount FROM (SELECT p.User, pe.Filename, m.ArtistFromFilename "; 
				sSQL = sSQL + "from Playlist p, PlaylistEntry pe, Music m WHERE p.Type = 'MIX' AND p.ROW_ID = pe.Playlist_ID  ";
				sSQL = sSQL + "and pe.Filename = m.Filename) a GROUP BY a.User, a.ArtistFromFilename) y where y.User =  ";
				sSQL = sSQL + "x.User and y.FileCount < x.One) as Two FROM (SELECT z.User, z.ArtistFromFilename,  ";
				sSQL = sSQL + "z.FileCount, (SELECT Max(y.FileCount) from (SELECT a.User, a.ArtistFromFilename,  ";
				sSQL = sSQL + "COUNT(a.Filename) as FileCount FROM (SELECT p.User, pe.Filename, m.ArtistFromFilename  ";
				sSQL = sSQL + "from Playlist p, PlaylistEntry pe, Music m WHERE p.Type = 'MIX' AND p.ROW_ID = pe.Playlist_ID  ";
				sSQL = sSQL + "and pe.Filename = m.Filename) a GROUP BY a.User, a.ArtistFromFilename) y where y.User =  ";
				sSQL = sSQL + "z.User) as One From (SELECT a.User, a.ArtistFromFilename, COUNT(a.Filename) as  ";
				sSQL = sSQL + "FileCount FROM (SELECT p.User, pe.Filename, m.ArtistFromFilename from Playlist p,  ";
				sSQL = sSQL + "PlaylistEntry pe, Music m WHERE p.Type = 'MIX' AND p.ROW_ID = pe.Playlist_ID and pe.Filename  ";
				sSQL = sSQL + "= m.Filename) a GROUP BY a.User, a.ArtistFromFilename) z) x) w) v ";
				sSQL = sSQL + "where v.FileCount = v.One OR v.FileCount = v.Two OR v.FileCount = v.Three ";
				sSQL = sSQL + "ORDER BY v.User, v.FileCount DESC ";	
			} //end Top Three User Artist
			
			if (sSelected.equals("Top Three Tags Per User")) {			
				sSQL = "SELECT '---' as NoKey, v.User, v.Tag, v.TagCount as Count FROM ";
				sSQL = sSQL + "(SELECT w.User, w.Tag, w.TagCount, w.One, w.Two,  (SELECT  ";
				sSQL = sSQL + "Max(y.TagCount) from (SELECT t.User, t.Tag, Count(t.Filename) as TagCount From Tags t GROUP BY t.User, t.Tag) y where y.User = w.User and "; 
				sSQL = sSQL + "y.TagCount < w.Two) as Three FROM (SELECT x.User, x.Tag, x.TagCount,  ";
				sSQL = sSQL + "x.One, (SELECT Max(y.TagCount) from (SELECT t.User, t.Tag, Count(t.Filename) as TagCount From Tags t GROUP BY t.User, t.Tag) y where y.User =  ";
				sSQL = sSQL + "x.User and y.TagCount < x.One) as Two FROM (SELECT z.User, z.Tag,  ";
				sSQL = sSQL + "z.TagCount, (SELECT Max(y.TagCount) from (SELECT t.User, t.Tag, Count(t.Filename) as TagCount From Tags t GROUP BY t.User, t.Tag) y where y.User = "; 
				sSQL = sSQL + "z.User) as One From (SELECT t.User, t.Tag, Count(t.Filename) as TagCount From Tags t GROUP BY t.User, t.Tag) z) x) w) v ";
				sSQL = sSQL + "where v.TagCount = v.One OR v.TagCount = v.Two OR v.TagCount = v.Three ";
				sSQL = sSQL + "ORDER BY v.User, v.TagCount DESC ";	
			} //end Top Three User Tags
			
			if (sSelected.equals("Most Common Tags")) {			
				sSQL = "SELECT '---' as NoKey, Tag, Count(Filename) as Count FROM Tags GROUP BY Tag Order By Count DESC";	
			} //end most common tags
			
			if (sSelected.equals("Count of Playlists by User by Type")) {			
				sSQL = "SELECT '---' as NoKey, p.User, p.Type, Count(p.Title) as Count from Playlist p GROUP BY p.User, p.Type ORDER BY p.User, Count DESC";	
			} //end playlist x user x type
			
			if (sSelected.equals("Genre Breakdown")) {			
				sSQL = "SELECT '---' as NoKey, b.Genre, Count(b.Filename) as Count from ";
				sSQL = sSQL + "(SELECT a.Filename, (Case ISNULL(a.NewGenre) When 1 then a.Genre else a.NewGenre end) as Genre from Music a where a.Genre > '' or a.newgenre > '') b GROUP ";
				sSQL = sSQL + "By b.Genre having b.genre <> 'NULL' Order By Count Desc;";	
			} //end genre breakdown
			
			if (sSelected.equals("Files With New Information")) {			
				sSQL = "SELECT Filename as TheKey, Filename, NewTitle, NewArtist, NewYear, NewGenre, NewTrack, NewAlbum, NewComment from Music where NewTitle is not null ";
				sSQL = sSQL + " or  NewArtist is not null or NewYear is not null or NewGenre is not null or NewTrack is not null or NewAlbum is not null or NewComment is not null ORDER BY Filename";	
			} //end files with new information
			
			if (sSelected.equals("Albums With Missing Year/Track Info")) {			
				sSQL = "SELECT '---' as NoKey, c.ArtistFromFilename as Artist, c.Album, c.Count as Count FROM ";
				sSQL = sSQL + "(SELECT b.ArtistFromFilename, b.Album, Count(b.Filename) as Count FROM ";
				sSQL = sSQL + "(SELECT a.Filename, a.ArtistFromFilename, a.Album from Music a Where a.Album > '' and (a.Track = '' or a.Year = '')) b ";
				sSQL = sSQL + "GROUP BY b.ArtistFromFilename, b.Album HAVING Count > 6 and b.Album not like 'Misc%') c ";
				sSQL = sSQL + "ORDER BY c.ArtistFromFilename, c.Count DESC";
			} //end albums missing info
			
			if (sSelected.equals("Artists With No Assigned Genre")) {			
				sSQL = "SELECT '---', g.ArtistFromFilename as Artist, Count(m.Filename) as Count FROM Music m, GenreMatch g where m.ArtistFromFilename = ";
				sSQL = sSQL + " g.ArtistFromFilename and g.Genre = '' GROUP BY g.ArtistFromFilename having count > 3 ORDER BY Count DESC";
			} //end artist missing genre
			
			if (sSelected.equals("Tracks Without Albums Per Artist")) {			
				sSQL = "SELECT '---', b.Artist, b.Count FROM (SELECT a.ArtistFromFilename as Artist, Count(a.Filename) as Count from Music a WHERE (Album is null or Album = '') GROUP BY ";
				sSQL = sSQL + "ArtistFromFilename) b WHERE b.Count > 6 Order By b.Count DESC;";
			} //end artist missing genre
	
			try {
				Vector columnNames = new Vector();
				Vector data = new Vector();
				stSQL = coDB.createStatement();
				rsResults = stSQL.executeQuery(sSQL);
	
				ResultSetMetaData mdResults = rsResults.getMetaData();
				int iColumnCount = mdResults.getColumnCount();
				
				if (rsResults.first()) {System.out.println("Records Returned");}
				
				//get ColumnNames
				for(int i=1; i <= iColumnCount ;i++){
					columnNames.addElement( mdResults.getColumnName(i) );
				}
				
				//get data
				boolean bContinue = rsResults.first();
				while (bContinue)
	            {
	                Vector row = new Vector(iColumnCount);
	                for (int i = 1; i <= iColumnCount; i++)
	                {
	                    row.addElement( rsResults.getObject(i) );
	                }
	                data.addElement( row );
	                bContinue = rsResults.next();
	            }
				
		        rsResults.close();
	            stSQL.close();
	            AndyGrid tbTemp = new AndyGrid(data, columnNames); 
	            setResultsTable(tbTemp);
				
			} // end try
			catch (SQLException e) {
				e.printStackTrace();
				System.out.println("SQL Error: " + e.getMessage());
				Vector columnNames = new Vector();
				Vector data = new Vector();
				Vector row = new Vector(1);
				columnNames.addElement("X");
				columnNames.addElement("SQL ERROR");
				row.addElement("!");
				row.addElement(e.getMessage());
				data.addElement(row);
				AndyGrid tbTemp = new AndyGrid(data, columnNames); 
	            	this.setResultsTable(tbTemp);
			} //end exception
		} // end if edited
	}

    private void setResultsTable(AndyGrid _tbResults) {
		tbResults = _tbResults;
		System.out.println((tbResults.getColumnCount()) + " columns");
		System.out.println((tbResults.getRowCount()) + " rows");
	    this.remove(spResults);
	    tbResults.formatGrid();
	    tbResults.getModel().addTableModelListener(this);
	    spResults.add(tbResults);
		spResults = new JScrollPane(tbResults);
	    spResults.setMinimumSize(new Dimension(700,400));
		spResults.setSize(new Dimension(700,400));
		spResults.setPreferredSize(new Dimension(700,400));
	    //spResults.setHorizontalScrollBarPolicy(32);
	    //spResults.setVerticalScrollBarPolicy(22);
	    this.add(spResults,cStatus);
	    this.repaint();
	    this.revalidate();
	    spResults.requestFocus();
	    tbResults.requestFocus();
	}

	public void tableChanged(TableModelEvent e) {
		int row = e.getFirstRow();
        	int column = e.getColumn();
        	if ( column != 0) {
	        	TableModel model = (TableModel)e.getSource();
	        tbResults.captureUpdate(row, column, (String) model.getValueAt(row, column));
        } //cannot edit key column
    }
    
} //END CLASS
