/*********************************
 * Andy's Super Awesome Reusable Java grid
 *********************************/

/**
 * Music Management Client 1.0
 * 
 * MODULE: AndyGrid
 * 
 * AUTHOR: Andy Maskin
 * 
 * PURPOSE: This class does a whole lot of heavy lifting for the program. 
 * It manages all of the grids in the application and does validations.
 * It maintains a 2D Vector of itself, so that in the event of invalid data,
 * it can revert a given cell to what it was before the invalid entry
 *
 */

import java.awt.Color;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.TableColumn;

public class AndyGrid extends JTable {

	private static final long serialVersionUID = 1L;
	public static Connection coDB = MMCObject.coDB;
	
	
	// CONSTRUCTORS
	//public AndyGrid(Vector data, Vector columnNames, Connection _coDB) {super(data, columnNames); coDB = _coDB;}
	public AndyGrid(Vector data, Vector columnNames) {super(data, columnNames);}
	public AndyGrid(int i, int j) {super(i,j);}
	public AndyGrid() {super();}
	
	Vector localData;	//used to save values for UNDO purposes
	
	//COMBO BOXES - TEXT FIELDS - CHECK BOXES
	JComboBox cmbGenre = MMCObject.cmbGenre;
	JComboBox cmbOrigArtist = MMCObject.cmbAFFN;
	JComboBox cmbPlaylistType = new JComboBox ();
	JTextField tfEditableCell = new JTextField();
	JTextField tfNonEditableCell = new JTextField();
	JTextField tfKeyCell = new JTextField();
	JCheckBox chEditableCheckbox = new JCheckBox();
	JCheckBox chNonEditableCheckbox = new JCheckBox();
	
	{
	}
	
	public void formatGrid () {
		
		setAutoResizeMode(0);
		setGridColor(Color.gray);
		tfKeyCell.setBackground(Color.BLACK);
		tfKeyCell.setForeground(Color.BLACK);
		tfEditableCell.setEditable(true);
		tfNonEditableCell.setEditable(false);
		chEditableCheckbox.setFocusable(true);
		chNonEditableCheckbox.setFocusable(false);
		cmbPlaylistType.addItem("ALB");
		cmbPlaylistType.addItem("MIX");
		cmbPlaylistType.setEditable(false);
		
		
		//Listeners
		this.getModel().addTableModelListener(this);
         
         //Declarations
		int iColumnCount = this.getColumnCount();
		TableColumn currColumn = null;
		String sColName;
		for (int i = 0; i < iColumnCount; i++) {
			currColumn = this.getColumnModel().getColumn(i);
			sColName = this.getColumnName(i);
			// Error Handler message
			if (sColName.equals("SQL ERROR")) {
				currColumn.setWidth(400);
			}
			//Non-Editable Columns
			if (sColName.equals("Title") || sColName.equals("Album")) {
				currColumn.setWidth(150);
				currColumn.setCellEditor(new DefaultCellEditor(tfNonEditableCell));
			}
			if (sColName.equals("Artist") || sColName.equals("Genre") || 
					sColName.equals("OldGenre") || sColName.equals("User") ||
					sColName.equals("GM_Artist")  || sColName.equals("A_Artist") || 
					sColName.equals("ArchiveFolder") ) {
				currColumn.setWidth(120);
				currColumn.setCellEditor(new DefaultCellEditor(tfNonEditableCell));
			}
			if (sColName.equals("Filename") || sColName.equals("FilenameWithPath")
					|| sColName.equals("File") || sColName.equals("PL_File")
					|| sColName.equals("Comment") || sColName.equals("Tag")) {
				currColumn.setWidth(180);
				currColumn.setCellEditor(new DefaultCellEditor(tfNonEditableCell));
			}
			if (sColName.equals("ArtistFromFilename") || sColName.equals("SongFromFilename") 
					|| sColName.equals("Track") || sColName.equals("Length")
					|| sColName.equals("Year") || sColName.equals("Count")) {
				currColumn.setWidth(50);
				currColumn.setCellEditor(new DefaultCellEditor(tfNonEditableCell));
			}
			//Editable Columns
			if (sColName.equals("NewGenre") || sColName.equals("GM_Genre") || sColName.equals("GT_Genre")) {
				currColumn.setWidth(100);
				currColumn.setCellEditor(new DefaultCellEditor(cmbGenre));
			}
			if (sColName.equals("Active")) {
				currColumn.setWidth(30);
				currColumn.setCellEditor(new DefaultCellEditor(chEditableCheckbox));
			}
			if (sColName.equals("OrigArtist")) {
				currColumn.setWidth(100);
				currColumn.setCellEditor(new DefaultCellEditor(cmbOrigArtist));
			}
			if (sColName.equals("PL_Type")) {
				currColumn.setWidth(60);
				currColumn.setCellEditor(new DefaultCellEditor(cmbPlaylistType));
			}
			if (sColName.equals("PL_Artist")) {
				currColumn.setWidth(100);
				currColumn.setCellEditor(new DefaultCellEditor(cmbOrigArtist));
			}
			if (sColName.equals("NewTitle") || sColName.equals("NewArtist") 
					|| sColName.equals("PL_Title")
					|| sColName.equals("NewAlbum") || sColName.equals("NewComment")) {
				currColumn.setWidth(150);
				currColumn.setCellEditor(new DefaultCellEditor(tfEditableCell));
			}
			if (sColName.equals("NewYear") || sColName.equals("PL_Year")
					|| sColName.equals("NewTrack")  || sColName.equals("PL_Track")) {
				currColumn.setWidth(70);
				currColumn.setCellEditor(new DefaultCellEditor(tfEditableCell));
			}
			
			//Set preferred width to newly defined sizes.
			currColumn.setPreferredWidth(currColumn.getWidth());
		}
		
		//Hide primary key column
		currColumn = this.getColumnModel().getColumn(0);
		currColumn.setWidth(0);
		currColumn.setMaxWidth(0);
		currColumn.setResizable(false);
		currColumn.setCellEditor(new DefaultCellEditor(tfKeyCell));
		
		this.setDragEnabled(false);
		
		//Populate localData
		vPopulateLocalData();
		
	} //end formatGrid
	
	//### PROCESSING EDITS ###
	public void vPopulateLocalData() {
		// This method captures the contents of the table into a vector
		// the reason for this is that if an invalid change is made
		// the old value can be recovered from this vector
		// and the change can be reverted

		int iRow;
		int iColumn;
		localData = new Vector();
		
        for (iRow = 0; iRow < this.getModel().getRowCount(); iRow++)
        {
    			Vector row = new Vector();
            	for (iColumn = 0; iColumn < this.getModel().getColumnCount(); iColumn++)
            {
            		if (this.getModel().getValueAt(iRow, iColumn) == null) {
            			row.addElement("");
            		} else {
            			row.addElement(this.getModel().getValueAt(iRow, iColumn).toString());
            		}
            } //end build rows of coordinate/data info
            	localData.addElement(row);
        } //end add each row
        
	} //end populateLocalData
	
	// Used to retrieve previous value of a given cell
	public String sRetrieveLocalData(int iRow, int iColumn) {
		String sValue = null;
		sValue = (String) ((Vector) localData.elementAt(iRow)).elementAt(iColumn);
		if (sValue == null) {sValue = "";};
		return sValue;
	}
	
	public boolean bFindMatchInColumn (int iColumn, String sValue){
		boolean bFound = false;
		for (int iRow=0; iRow < this.getModel().getRowCount(); iRow++) {
			if (this.getModel().getValueAt(iRow, iColumn).toString().equals(sValue)) {bFound = true;}
		}
		
		return bFound;
	}
	
	
	// ==== DELETIONS =======
	public void vDeleteRecord() {
		String sSQL ="";
		String sKey ="";
		String sColName;
		int iTargetRow = this.getSelectedRow();
		
		if (iTargetRow >= 0) {
			sKey = (String) this.getValueAt(iTargetRow,0).toString();
			sColName = this.getColumnName(1);
		
			// PLAYLIST ENTRY TABLE
			if (sColName.equals("PL_File")) {
				sSQL = "DELETE FROM PlaylistEntry WHERE ROW_ID = ";
				sSQL = sSQL + sKey;
				executeSQL(sSQL,0,0);
			} //end delete from playlist entry
			
			//TAGS TABLE
			if (sColName.equals("Tag")) {
				sSQL = "DELETE FROM Tags WHERE ROW_ID = ";
				sSQL = sSQL + sKey;
				executeSQL(sSQL,0,0);
			} //end delete from tag table
			
			//PLAYLIST TABLE
			if (sColName.equals("PL_Artist")) {
				sSQL = "DELETE FROM Playlist WHERE ROW_ID = ";
				sSQL = sSQL + sKey;
				executeSQL(sSQL,0,0);
			} //end delete from playlist entry
			
			//GENRE TABLE
			if (sColName.equals("Genre")) {
				sSQL = "DELETE FROM Genre WHERE Genre = \"";
				sSQL = sSQL + sKey + "\"";
				executeSQL(sSQL,0,0);
				
				//Clean up references in other tables
				sSQL = "UPDATE GenreMatch SET Genre = \"\" WHERE Genre = \"";
				sSQL = sSQL + sKey + "\"";
				executeSQL(sSQL,0,0);
				
				sSQL = "INSERT INTO GenreTranslate (OldGenre, Genre) VALUES (\"";
				sSQL = sSQL + sKey + "\", \"\") ON DUPLICATE KEY UPDATE OldGenre = \"\" ";
				executeSQL(sSQL,0,0);
				
			} //end delete from playlist entry
			
		} //end if target row found
	} //end DELETIONS
	
	public void captureUpdate(int iRow, int iColumn, String sValue) {
		if (sValue == null) {sValue = "";}
		// VALIDATE & See if a change has been made
		if (bValidated(iRow, iColumn, sValue) == true) {
			if (sRetrieveLocalData(iRow, iColumn).equals(sValue) == false) {
				vProcessUpdate(iRow, iColumn, sValue);
				vPopulateLocalData(); 		//Refresh Local copy of data
			} //end if value is new
		} else {
			undoUpdate(iRow, iColumn);
		} //end if validated
	} //end captureUpdate
	
	// UNDO a change made to the grid
	public void undoUpdate(int iRow, int iColumn) {
		System.out.println("Invalid entry! Getting old value: " + sRetrieveLocalData(iRow, iColumn));
		if (bValidated(iRow, iColumn, sRetrieveLocalData(iRow, iColumn)) == true) {
			// If previous value is valid, then use it
			this.setValueAt(sRetrieveLocalData(iRow, iColumn),iRow, iColumn);
		} else {
			// if previous value is invalid, set the cell to -NULL-
			this.setValueAt("-NULL-",iRow, iColumn);
		}
	} //end undoUpdate

	
	// ******* VALIDATION ************************
	public boolean bValidated (int iRow, int iColumn, String sValue){
		boolean bFlag = true;
		if (sValue.equals("-INVALID-")) {bFlag = false; }
		
		String sColName = this.getColumnName(iColumn);
		
		if (sValue.equals("") == false && ( sColName.equals("NewYear") || sColName.equals("NewTrack"))) {
			int iTest = 0;
			Calendar cal = new GregorianCalendar();
			try {
				iTest = Integer.parseInt(sValue);
				if (iTest == 9999) {System.out.println(iTest);}
			} catch (Exception e){
    				System.out.println("Error! Track must be an Integer!");
    				bFlag = false;
			} //end Int check
			if (sColName.equals("NewYear")) {
				if (iTest < 1900 || iTest > cal.get(Calendar.YEAR) + 1) {
					System.out.println("Error! Invalid Year!");
    					bFlag = false;
				}
			} //end year check
		} //end check of newyear and newtrack
		
		if ((sColName.equals("NewArtist") || sColName.equals("NewTitle")) &&
				sValue.equals("-NULL-")) {
    				System.out.println("Error! Artist And Title cannot be blank!");
    				bFlag = false;
		} //end check of newartist and newTitle
		
		return bFlag;
	}
	
	// ******* PROCESS UPDATES *******************
	public void vProcessUpdate (int iRow, int iColumn, String sValue){
		
		String sColName;
		String sSQL ="";
		String sKey;
		
		sColName = this.getColumnName(iColumn);
		sKey = (String) this.getValueAt(iRow,0).toString();
		
		//Catch NULLs
		if (sValue.equals("")) {
			sValue = "NULL";
		} else {
			sValue = "\"" + sValue + "\"";
		}
		
		//MUSIC TABLE
		if (sColName.equals("NewTitle") || sColName.equals("NewArtist") 
				|| sColName.equals("NewTrack") || sColName.equals("NewGenre") 
				|| sColName.equals("NewAlbum") || sColName.equals("NewComment")
				|| sColName.equals("NewYear") || sColName.equals("OrigArtist")) {
			sSQL = "UPDATE Music SET " + sColName + "=" + sValue + " WHERE ";
			sSQL = sSQL + "Filename = \"" + sKey + "\"";
			System.out.println(sSQL);
			executeSQL(sSQL, iRow,  iColumn);
		}
		
		//PLAYLIST TABLE
		if (sColName.equals("PL_Title")) {
			sSQL = "UPDATE Playlist SET Title = " + sValue + " WHERE ";
			sSQL = sSQL + "ROW_ID = " + sKey;
			System.out.println(sSQL);
			executeSQL(sSQL, iRow,  iColumn);
		}
		if (sColName.equals("PL_Year")) {
			sSQL = "UPDATE Playlist SET Year = " + sValue + " WHERE ";
			sSQL = sSQL + "ROW_ID = " + sKey;
			System.out.println(sSQL);
			executeSQL(sSQL, iRow,  iColumn);
		}
		if (sColName.equals("PL_Artist")) {
			sSQL = "UPDATE Playlist SET ArtistFromFilename = " + sValue + " WHERE ";
			sSQL = sSQL + "ROW_ID = " + sKey;
			System.out.println(sSQL);
			executeSQL(sSQL, iRow,  iColumn);
		}
		
		//PLAYLIST ENTRY TABLE
		if (sColName.equals("PL_Track")) {
			sSQL = "UPDATE PlaylistEntry SET Track = " + sValue + " WHERE ";
			sSQL = sSQL + "ROW_ID = " + sKey;
			System.out.println(sSQL);
			executeSQL(sSQL, iRow,  iColumn);
		}
		
		//GENRE-MATCH TABLE
		if (sColName.equals("GM_Genre")) {
			sSQL = "UPDATE GenreMatch SET Genre = " + sValue + " WHERE ";
			sSQL = sSQL + "ArtistFromFilename = \"" + sKey + "\"";
			System.out.println(sSQL);
			executeSQL(sSQL, iRow,  iColumn);
		}
		
		//GENRE-TRANSLATE TABLE
		if (sColName.equals("GT_Genre")) {
			sSQL = "UPDATE GenreTranslate SET Genre = " + sValue + " WHERE ";
			sSQL = sSQL + "OldGenre = \"" + sKey + "\"";
			System.out.println(sSQL);
			executeSQL(sSQL, iRow,  iColumn);
		}
		
		//COVER-MATCH TABLE
		if (sColName.equals("OrigArtist")) {
			if (sValue.equals("NULL")) {sValue = "";}
			if (sValue.length() >= 1) {
				//SEND INSERT w/Update to CoverMatch Table
				sSQL = "INSERT INTO CoverMatch (Filename, OrigArtist) VALUES (\"" + sKey + "\"";
				sSQL = sSQL + ", " + sValue + ") ";
				sSQL = sSQL + "ON DUPLICATE KEY UPDATE OrigArtist = " + sValue;
			} else {
				//SEND DELETE to coverMatch Table
				sSQL = "DELETE FROM CoverMatch Where Filename = \"" + sKey + "\"";
			} // end 
			System.out.println(sSQL);
			executeSQL(sSQL, iRow,  iColumn);
			executeSQL("COMMIT", 0,  0);
		} //end cover match logic
	}// end vProcessUpdate
	
	public void executeSQL(String sSQL, int iRow, int iColumn){
		Statement stSQL;
		String sAudit;
		
		sAudit = "INSERT INTO AuditTrail (Action, User) VALUES(";
		sAudit = sAudit + "\"" + sSQL.replace('"','\'') + "\", \"" + MMCObject.sUsername + "\")";
		
		try {
		stSQL = coDB.createStatement();
		stSQL.execute(sSQL);
		stSQL.execute(sAudit);
		stSQL.close();
		}// end try
		catch (SQLException e) {
			System.out.println("SQL Error: " + e.getMessage());
			this.undoUpdate(iRow, iColumn);
		} //end exception
	}

} //end AnyGrid class 
