import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JPanel;

/**
 * Music Management Client 1.0
 * 
 * MODULE: CommandsPanel
 * 
 * AUTHOR: Andy Maskin
 * 
 * PURPOSE: This panel is where the buttons that launch assorted batch commands are displayed
 *
 */

public class CommandsPanel extends JPanel implements  ActionListener {

	private static final long serialVersionUID = 1L;
	public static Connection coDB;
	
	// GRAPHIC ELEMENTS
	public static JButton btnRefresh = new JButton("Refresh Picklists");
	public static JButton btnGenre = new JButton("Run Genre Fix Logic");
	public static JButton btnArtist = new JButton("Run Artist Fix Logic");
	public static JButton btnGenerate = new JButton("Generate DOS Batch File");
	public static JButton btnRandList = new JButton("Generate Random 850 List");
	
	public static JFileChooser fc = new JFileChooser();
	
	File dDirectory;
	String sDirectory;
	
    
    GridBagConstraints cStatus = new GridBagConstraints();
    GridBagConstraints cLabels = new GridBagConstraints();
    GridBagConstraints cFields = new GridBagConstraints();
	
    {
    		//PASSED-IN VALUES
    		coDB = MMCObject.coDB;
    		cStatus = MMCObject.cStatus;
    		cLabels = MMCObject.cLabels;
    		cFields = MMCObject.cFields;
    	
    		fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		 btnRefresh.addActionListener(this); 
		 btnGenre.addActionListener(this); 
		 btnArtist.addActionListener(this); 
		 btnGenerate.addActionListener(this); 
		 btnRandList.addActionListener(this); 
    		
    		//LAYOUT STUFF
		this.setLayout(new GridBagLayout());
		this.add(btnRefresh,cLabels);
	    	this.add(btnGenre,cFields);
	    	this.add(btnArtist,cLabels);
	    	this.add(btnGenerate,cFields);
	    	this.add(btnRandList,cLabels);
	    
    }
    
    public void actionPerformed(ActionEvent arg0) {
		//Refresh Button
		if (arg0.getActionCommand().equals("Refresh Picklists")) {
			vRefreshPicklists();
		} //end Refresh
		if (arg0.getActionCommand().equals("Run Genre Fix Logic")) {
			vGenreFix();
		} //end GenreFix
		if (arg0.getActionCommand().equals("Run Artist Fix Logic")) {
			vArtistFix();
		} //end ArtistFix
		if (arg0.getActionCommand().equals("Generate DOS Batch File")) {
			vDOSBatch();
		} //end DOSBatch
		if (arg0.getActionCommand().equals("Generate Random 850 List")) {
			vRandList();
		} //end RandList
	}

    public void vRefreshPicklists (){
    		MMCObject.vBuildPicklists();
    } //end vRefreshPicklists
    
	public void vGenreFix (){
		String sSQL = "";
		sSQL = "INSERT INTO GenreTranslate (OldGenre) ";
		sSQL = sSQL + "SELECT DISTINCT a.Genre as InvalidGenre FROM ";
		sSQL = sSQL + "(Music a LEFT JOIN Genre b ON a.Genre = b.Genre) LEFT JOIN  ";
		sSQL = sSQL + "GenreTranslate c ON a.Genre = c.OldGenre ";
		sSQL = sSQL + "where a.Genre is not null and a.Genre <> '' and  ";
		sSQL = sSQL + "b.Genre is null and c.OldGenre is null; ";
	    	
		MMCObject.executeSQL(sSQL);
		
		sSQL = "INSERT INTO GenreMatch (ArtistFromFileName)  ";
		sSQL = sSQL + "SELECT DISTINCT a.ArtistFromFilename as NewArtist FROM  ";
		sSQL = sSQL + "Music a LEFT JOIN GenreMatch b ON a.ArtistFromFilename =  ";
		sSQL = sSQL + "b.ArtistFromFilename ";
		sSQL = sSQL + "where a.ArtistFromFilename is not null and b.ArtistFromFilename is null ";
		
		MMCObject.executeSQL(sSQL);
		
		sSQL = "UPDATE Music a INNER JOIN GenreMatch b  ";
		sSQL = sSQL + "ON a.ArtistFromFilename = b.ArtistFromFilename ";
		sSQL = sSQL + "SET a.NewGenre = b.Genre ";
		sSQL = sSQL + "WHERE a.Genre <> b.Genre and b.Genre <> '' ";
		
		MMCObject.executeSQL(sSQL);
		
	} //end vGenreFix
	
	public void vArtistFix (){
		String sSQL;
		
		sSQL = "DROP TABLE IF EXISTS Temp_ArtistCleanup";
		
		MMCObject.executeSQL(sSQL);
		
		sSQL = "CREATE TABLE Temp_ArtistCleanup (ArtistFromFilename varchar(100)  default '',";
		sSQL = sSQL + " NewArtist varchar(100)  default '') TYPE=MyISAM";
		
		MMCObject.executeSQL(sSQL);
		
		sSQL = "INSERT INTO Temp_ArtistCleanup (ArtistFromFilename, NewArtist) ";
		sSQL = sSQL + "SELECT DISTINCT g.ArtistFromFilename,  ";
		sSQL = sSQL + "(SELECT e.Artist FROM ";
		sSQL = sSQL + "(SELECT c.ArtistFromFilename, c.artist, (count(c.filename) + Rand()) as  ";
		sSQL = sSQL + "counter FROM ";
		sSQL = sSQL + "(SELECT b.ArtistFromFilename, (CASE a.NewArtist When '' then a.Artist When NOT(NULL) then a.NewArtist ";
		sSQL = sSQL + "else a.Artist end) as Artist, a.Filename FROM Music a, ";
		sSQL = sSQL + "(SELECT ArtistFromFilename from Music GROUP BY ArtistFromFilename ";
		sSQL = sSQL + "                having Count(DISTINCT Artist) > 1) b ";
		sSQL = sSQL + "                WHERE a.ArtistFromFilename = b.ArtistFromFilename) as c ";
		sSQL = sSQL + "GROUP BY c.ArtistFromFilename, c.artist) as e ";
		sSQL = sSQL + "WHERE e.ArtistFromFilename = g.ArtistFromFilename ";
		sSQL = sSQL + "ORDER BY e.counter DESC LIMIT 1) as NewArtist ";
		sSQL = sSQL + "FROM Music g";
		
		MMCObject.executeSQL(sSQL);
		
		sSQL = "DELETE FROM Temp_ArtistCleanup where NewArtist is null";
		
		MMCObject.executeSQL(sSQL);
		
		sSQL = "UPDATE Music a Set a.NewArtist = (SELECT b.NewArtist From Temp_ArtistCleanup b WHERE "; 
		sSQL = sSQL + "a.ArtistFromFilename = b.ArtistFromFilename) where a.NewArtist is null  or a.NewArtist = ''";
		
		MMCObject.executeSQL(sSQL);

		sSQL = "Update Music Set NewArtist = NULL Where Artist = NewArtist";
		
		MMCObject.executeSQL(sSQL);
		
		sSQL = "DROP TABLE IF EXISTS Temp_ArtistCleanup";
		
		MMCObject.executeSQL(sSQL);
	} //end vArtistFix
	
	public void vDOSBatch (){
		String sSQL = "";
		Statement stSQL;
		ResultSet rsResults;
		boolean bProceed;
		int returnVal = 0;
		
		bProceed = true;
		
		while(bProceed) {
			returnVal = fc.showSaveDialog(this);
			if (returnVal != JFileChooser.ERROR_OPTION) {bProceed = false;}
		}
        
         if (returnVal == JFileChooser.APPROVE_OPTION) {
             File file = fc.getSelectedFile();
             System.out.println("Saving: " + file.getPath());
             
             sSQL = "SELECT CONCAT('id3 -2 -a \"',(CASE NewArtist When '-NULL-' THEN NewArtist ELSE NewArtist END),'\" \"',FilenameWithPath,'\"') as x from Music where NewArtist > ''";
             sSQL = sSQL + " UNION";
             sSQL = sSQL + " SELECT CONCAT('id3 -2 -t \"',(CASE NewTitle When '-NULL-' THEN NewTitle ELSE NewTitle END),'\" \"',FilenameWithPath,'\"') as x from Music where NewTitle > ''";
             sSQL = sSQL + " UNION";
             sSQL = sSQL + " SELECT CONCAT('id3 -2 -l \"',(CASE NewAlbum When '-NULL-' THEN '' ELSE NewAlbum END),'\" \"',FilenameWithPath,'\"') as x from Music where NewAlbum > ''";
             sSQL = sSQL + " UNION";
             sSQL = sSQL + " SELECT CONCAT('id3 -2 -n \"',(CASE NewTrack When '-NULL-' THEN '' ELSE NewTrack END),'\" \"',FilenameWithPath,'\"') as x from Music where NewTrack > ''";
             sSQL = sSQL + " UNION";
             sSQL = sSQL + " SELECT CONCAT('id3 -2 -y \"',(CASE NewYear When '-NULL-' THEN '' ELSE NewYear END),'\" \"',FilenameWithPath,'\"') as x from Music where NewYear > ''";
             sSQL = sSQL + " UNION";
             sSQL = sSQL + " SELECT CONCAT('id3 -2 -g \"',(CASE NewGenre When '-NULL-' THEN '' ELSE NewGenre END),'\" \"',FilenameWithPath,'\"') as x from Music where NewGenre > ''";
             sSQL = sSQL + " UNION";
             sSQL = sSQL + " SELECT CONCAT('id3 -2 -c \"',(CASE NewComment When '-NULL-' THEN '' ELSE NewComment END),'\" \"',FilenameWithPath,'\"') as x from Music where NewComment > ''";
             
             //WRITE TO FILE
             try {
            	 
				FileWriter out = new FileWriter(file);
				String sCommand;
					
            	 	try {
						stSQL = coDB.createStatement();
						rsResults = stSQL.executeQuery(sSQL);
						
						
						boolean bContinue = rsResults.first();
						while (bContinue)
			            {
							sCommand = rsResults.getString(1);
							out.write(sCommand);
							out.write(10);
							out.write(13);
			                 bContinue = rsResults.next();
			            }
						
				        rsResults.close();
			            stSQL.close();
			            
			            System.out.println("File Written.");
						
					} catch (SQLException e) {
			             System.out.println("SQL Error!");
						e.printStackTrace();
					} //end try-catch Build 
					
				out.close();
			} catch (IOException e) {
	             System.out.println("Error Writing File!");
				e.printStackTrace();
			} //end try-catch write file
             
         } else {
        	 	System.out.println("Save command cancelled by user.");
         }			
		
	} //vDOSBatch
	
	public void vRandList (){
		String sSQL = "";
		Statement stSQL;
		ResultSet rsResults;
		boolean bProceed;
		int returnVal = 0;
		
		bProceed = true;
		
		while(bProceed) {
			returnVal = fc.showSaveDialog(this);
			if (returnVal != JFileChooser.ERROR_OPTION) {bProceed = false;}
		}
        
         if (returnVal == JFileChooser.APPROVE_OPTION) {
             File file = fc.getSelectedFile();
             System.out.println("Saving: " + file.getPath());
             
             sSQL = "SELECT g.Type, h.FilenameWithPath FROM ";
 			sSQL = sSQL + "(SELECT d.Filename, 'ALB' as Type ";
 			sSQL = sSQL + "FROM ";
 			sSQL = sSQL + "PlaylistEntry d, ";
 			sSQL = sSQL + "(SELECT b.ROW_ID, b.Title, b.ArtistFromFilename ";
 			sSQL = sSQL + "FROM (SELECT a.ROW_ID, a.Title, a.ArtistFromFilename, RAND() as "; 
 			sSQL = sSQL + "randomizer ";
 			sSQL = sSQL + "FROM (SELECT p.ROW_ID, p.ArtistFromFilename, p.Title, pe.Filename, "; 
 			sSQL = sSQL + "pe.Track from Playlist p, PlaylistEntry pe ";
			sSQL = sSQL + "where p.ROW_ID = pe.Playlist_ID and p.title > '' and p.type = 'ALB') a ";
			sSQL = sSQL + "GROUP BY a.Title, a.ArtistFromFilename HAVING Count(a.Filename) > 6 and  ";
			sSQL = sSQL + "Count(a.Filename)/Max(a.Track) >= .75) b ";
			sSQL = sSQL + "where b.ArtistFromFilename > '' ORDER BY b.Randomizer LIMIT 30) c ";
			sSQL = sSQL + "where d.Playlist_ID = c.ROW_ID ";
			sSQL = sSQL + "UNION ";
			sSQL = sSQL + "SELECT f.Filename, 'IND' as Type FROM ";
			sSQL = sSQL + "(SELECT e.Filename, Rand() as randomizer FROM Music e) f) g, ";
			sSQL = sSQL + "Music h ";
			sSQL = sSQL + "where h.Filename = g.Filename ";
			sSQL = sSQL + "ORDER BY g.Type LIMIT 850";
             sSQL = sSQL + "";
             
             //WRITE TO FILE
             try {
            	 
				FileWriter out = new FileWriter(file);
				String sCommand;
					
            	 	try {
						stSQL = coDB.createStatement();
						rsResults = stSQL.executeQuery(sSQL);
						
						
						boolean bContinue = rsResults.first();
						while (bContinue)
			            {
							sCommand = rsResults.getString(1) + "\t" + rsResults.getString(2);
							out.write(sCommand);
							out.write(10);
							out.write(13);
			                 bContinue = rsResults.next();
			            }
						
				        rsResults.close();
			            stSQL.close();
			            
			            System.out.println("File Written.");
						
					} catch (SQLException e) {
			             System.out.println("SQL Error!");
						e.printStackTrace();
					} //end try-catch Build 
					
				out.close();
			} catch (IOException e) {
	             System.out.println("Error Writing File!");
				e.printStackTrace();
			} //end try-catch write file
             
         } else {
        	 	System.out.println("Save command cancelled by user.");
         }			
		
	} //vRandList
    
} //END CLASS
