import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 * Music Management Client 1.0
 * 
 * MODULE: GenreTranslatePanel
 * 
 * AUTHOR: Andy Maskin
 * 
 * PURPOSE: This panel displays every invalid Genre that has passed through the system and allows 
 * the user to map each value to a valid genre. In cases where no GenreMatch value is available 
 * and a file and the file has an invalid genre, this table is used to assign a new genre to the file. 
 *
 */

public class GenreTranslatePanel extends JPanel implements TableModelListener {

	private static final long serialVersionUID = 1L;
	public static Connection coDB;
	
	// GRAPHIC ELEMENTS
    AndyGrid tbResults = new AndyGrid(0,0);
    JScrollPane spResults;
    
    GridBagConstraints cStatus = new GridBagConstraints();
    GridBagConstraints cLabels = new GridBagConstraints();
    GridBagConstraints cFields = new GridBagConstraints();
	
    {
    		//PASSED-IN VALUES
    		coDB = MMCObject.coDB;
    		cStatus = MMCObject.cStatus;
    		cLabels = MMCObject.cLabels;
    		cFields = MMCObject.cFields;
    	
    		//LAYOUT STUFF
		spResults = new JScrollPane(tbResults);
		tbResults.setPreferredScrollableViewportSize(new Dimension(700, 600));
		spResults.setMinimumSize(new Dimension(700,600));
		populateGrid();
    }
    
    public void populateGrid() {
	    	    Statement stSQL;
			ResultSet rsResults;
			String sSQL;
			
			sSQL = "SELECT OldGenre as OGKey, OldGenre, Genre as GT_Genre from GenreTranslate ORDER BY OldGenre";
			
			try {
				Vector columnNames = new Vector();
				Vector data = new Vector();
				stSQL = coDB.createStatement();
				rsResults = stSQL.executeQuery(sSQL);
	
				ResultSetMetaData mdResults = rsResults.getMetaData();
				int iColumnCount = mdResults.getColumnCount();
				
				if (rsResults.first()) {System.out.println("Records Returned");}
				
				//get ColumnNames
				for(int i=1; i <= iColumnCount ;i++){
					columnNames.addElement( mdResults.getColumnName(i) );
				}
				
				//get data
				boolean bContinue = rsResults.first();
				while (bContinue)
	            {
	                Vector row = new Vector(iColumnCount);
	                for (int i = 1; i <= iColumnCount; i++)
	                {
	                    row.addElement( rsResults.getObject(i) );
	                }
	                data.addElement( row );
	                bContinue = rsResults.next();
	            }
				
		        rsResults.close();
	            stSQL.close();
	            AndyGrid tbTemp = new AndyGrid(data, columnNames); 
	            setResultsTable(tbTemp);
				
			} // end try
			catch (SQLException e) {
				e.printStackTrace();
				System.out.println("SQL Error: " + e.getMessage());
				Vector columnNames = new Vector();
				Vector data = new Vector();
				Vector row = new Vector(1);
				columnNames.addElement("X");
				columnNames.addElement("SQL ERROR");
				row.addElement("!");
				row.addElement(e.getMessage());
				data.addElement(row);
				AndyGrid tbTemp = new AndyGrid(data, columnNames); 
	            	this.setResultsTable(tbTemp);
			} //end exception
	}

    private void setResultsTable(AndyGrid _tbResults) {
		tbResults = _tbResults;
		System.out.println((tbResults.getColumnCount()) + " columns");
		System.out.println((tbResults.getRowCount()) + " rows");
	    this.remove(spResults);
	    tbResults.formatGrid();
	    tbResults.getModel().addTableModelListener(this);
	    spResults.add(tbResults);
		spResults = new JScrollPane(tbResults);
	    spResults.setMinimumSize(new Dimension(700,400));
		spResults.setSize(new Dimension(700,400));
		spResults.setPreferredSize(new Dimension(700,400));
	    //spResults.setHorizontalScrollBarPolicy(32);
	    //spResults.setVerticalScrollBarPolicy(22);
	    this.add(spResults,cStatus);
	    this.repaint();
	    this.revalidate();
	    spResults.requestFocus();
	    tbResults.requestFocus();
	}

	public void tableChanged(TableModelEvent e) {
		int row = e.getFirstRow();
        	int column = e.getColumn();
        	if ( column != 0) {
	        	TableModel model = (TableModel)e.getSource();
	        tbResults.captureUpdate(row, column, (String) model.getValueAt(row, column));
        } //cannot edit key column
    }
    
} //END CLASS
