import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import javax.swing.JComboBox;

/**
 * Music Management Client 1.0
 * 
 * MODULE: MMCObject
 * 
 * AUTHOR: Andy Maskin
 * 
 * PURPOSE: This object stores values and other objects that are common across the whole application.
 * it gets build once a connection is established and passes the connection object to all of the panels
 * so that the connection only needs to be established once.
 * It also builds all of the comboBoxes (or in come cases, vectors) that involve heavy querying
 * of the database. Again, the purpose of this is so that it only has to happen once, at launch. 
 *
 */

public class MMCObject {
	
	public static Connection coDB;
	public static JComboBox cmbAFFN;
	public static JComboBox cmbGenre;
	public static JComboBox cmbPlaylistType;
	public static JComboBox cmbPossibleAnalysis;
	public static JComboBox cmbUsers;
	public static AComboBox acbPlaylist;
	
	public static Vector vAFFN;
	public static Vector vFilename;
	
	public static String sUsername;
	public static String sServer;
	public static String sDatabase;
	public static String sPassword;
	
	public static GridBagConstraints cStatus = new GridBagConstraints();
	public static GridBagConstraints cLabels = new GridBagConstraints();
	public static GridBagConstraints cFields = new GridBagConstraints();
	
	//CONSTRUCTOR
	public MMCObject(Connection _coDB, String _sUsername, String _sPassword, String _sDatabase, String _sServer) {
		
		//Capture Connection
		coDB = _coDB;
		sUsername = _sUsername;
		sPassword = _sPassword;
		sServer = _sServer;
		sDatabase = _sDatabase;
		
		vBuildPicklists();
		
		// BUILD GRIDBAG CONSTRAINTS
		cLabels.gridwidth = GridBagConstraints.RELATIVE; //next-to-last
		cLabels.fill = GridBagConstraints.NONE;      //reset to default
		cLabels.weightx = 0.0;                       //reset to default
		cLabels.weighty = 0.0;
		cLabels.ipadx = 5;
		cLabels.anchor = GridBagConstraints.FIRST_LINE_END;
		cLabels.insets = new Insets(10,30,3,3);
		cFields.gridwidth = GridBagConstraints.REMAINDER;     //end row
		cFields.fill = GridBagConstraints.NONE;
		cFields.weightx = 0.0;
		cFields.weighty = 0.0;
		cFields.ipadx = 5;
		cFields.anchor = GridBagConstraints.FIRST_LINE_START;
		cFields.insets = new Insets(10,3,3,30);
		cStatus.gridwidth = GridBagConstraints.REMAINDER; //last
	    cStatus.anchor = GridBagConstraints.LAST_LINE_START;
	    cStatus.weightx = 1.0;
	    cStatus.weighty = 1.0;
		
	} //END CONSTRUCTOR
	
	//EXECUTE SQL FOR PANELS WITHOUT GRIDS
	public static boolean executeSQL(String sSQL){
		boolean bResult;
		bResult = true;
		
		Statement stSQL;
		
		String sAudit;
		
		sAudit = "INSERT INTO AuditTrail (Action, User) VALUES(";
		sAudit = sAudit + "\"" + sSQL.replace('"','\'') + "\", \"" + MMCObject.sUsername + "\")";
		
		try {
		stSQL = coDB.createStatement();
		stSQL.execute(sSQL);
		stSQL.execute(sAudit);
		stSQL.close();
		}// end try
		catch (SQLException e) {
			System.out.println("SQL Error: " + e.getMessage());
			bResult = false;
		} //end exception
		
		return bResult;
	} //end executeSQL method
	
	public static void vBuildPicklists() {
		//####### Build ComboBoxes ########
		System.out.println("Building Comboboxes ...");
		cmbAFFN = new JComboBox();
		cmbGenre = new JComboBox();
		cmbPlaylistType = new JComboBox();
		cmbPossibleAnalysis = new JComboBox();
		cmbUsers = new JComboBox();
		acbPlaylist = new AComboBox();
		Statement stSQL;
		ResultSet rs;
		
		//cmbAFFN
		try {
			stSQL = coDB.createStatement();
			rs = stSQL.executeQuery("SELECT DISTINCT ArtistFromFilename from Music ORDER BY ArtistFromFilename");
			cmbAFFN.addItem("");
			while (rs.next()){cmbAFFN.addItem(rs.getObject(1)); }
			rs.close();
			stSQL.close();
			cmbAFFN.setEditable(false);
		} catch (SQLException e) {
			e.printStackTrace();
			System.out.println("SQL Error: " + e.getMessage());
		}
		
		//cmbGenre
		cmbGenre.addItem("");
		try {
			stSQL = coDB.createStatement();
			rs = stSQL.executeQuery("SELECT DISTINCT Genre from Genre WHERE Active = 'Y' ORDER BY Genre");
			while (rs.next()){cmbGenre.addItem(rs.getObject(1));}
			rs.close();
			stSQL.close();
			cmbGenre.setEditable(false);
		} catch (SQLException e) {
			e.printStackTrace();
			System.out.println("SQL Error: " + e.getMessage());
		}
		
		//cmbPlaylistType
		cmbPlaylistType.addItem("Any");
		cmbPlaylistType.addItem("ALB");
		cmbPlaylistType.addItem("MIX");

		//cmbPossibleAnalysis
		cmbPossibleAnalysis.addItem("");
		cmbPossibleAnalysis.addItem("Possible Duplicates");
		cmbPossibleAnalysis.addItem("File Count By Artist");
		cmbPossibleAnalysis.addItem("Top Three Mix Artists Per User");
		cmbPossibleAnalysis.addItem("Top Three Mix Genres Per User");
		cmbPossibleAnalysis.addItem("Top Three Tags Per User");
		cmbPossibleAnalysis.addItem("Most Common Tags");
		cmbPossibleAnalysis.addItem("Count of Playlists by User by Type");
		cmbPossibleAnalysis.addItem("Genre Breakdown");
		cmbPossibleAnalysis.addItem("Files With New Information");
		cmbPossibleAnalysis.addItem("Albums With Missing Year/Track Info");
		cmbPossibleAnalysis.addItem("Artists With No Assigned Genre");
		cmbPossibleAnalysis.addItem("Tracks Without Albums Per Artist");
		
		//cmbUsers
		cmbUsers.addItem("Any");
		try {
			stSQL = coDB.createStatement();
			rs = stSQL.executeQuery("SELECT DISTINCT User from Playlist ORDER BY User");
			while (rs.next()){cmbUsers.addItem(rs.getObject(1));}
			rs.close();
			stSQL.close();
			cmbUsers.setEditable(false);
		} catch (SQLException e) {
			e.printStackTrace();
			System.out.println("SQL Error: " + e.getMessage());
		}
		
		//acbAFFN
		vAFFN = new Vector();
		try {
			stSQL = coDB.createStatement();
			rs = stSQL.executeQuery("SELECT DISTINCT ArtistFromFilename from Music  ORDER BY ArtistFromFilename");
			while (rs.next()){vAFFN.addElement(rs.getObject(1));}
			rs.close();
			stSQL.close();
		} catch (SQLException e) {
			e.printStackTrace();
			System.out.println("SQL Error: " + e.getMessage());
		}
		
		//acbFilename
		vFilename = new Vector();
		try {
			stSQL = coDB.createStatement();
			rs = stSQL.executeQuery("SELECT DISTINCT Filename from Music ORDER BY Filename");
			while (rs.next()){vFilename.addElement(rs.getObject(1));}
			rs.close();
			stSQL.close();
		} catch (SQLException e) {
			e.printStackTrace();
			System.out.println("SQL Error: " + e.getMessage());
		}
		
		//acbPlaylist
		ComboItem newItem = new ComboItem(0,"");
		try {
			acbPlaylist.addItem(newItem);
			stSQL = coDB.createStatement();
			rs = stSQL.executeQuery("SELECT ROW_ID, CONCAT(TYPE,if(isNull(ArtistFromFilename),'',' - '),ArtistFromFilename,' - ',Title) as PL From Playlist  ORDER BY PL");
			while (rs.next()){
				long lID;
				lID = Long.parseLong(rs.getObject(1).toString());
				newItem = new ComboItem(lID,(String) rs.getObject(2));
				acbPlaylist.addItem(newItem);
			}
			rs.close();
			stSQL.close();
			acbPlaylist.setEditable(true);
		} catch (SQLException e) {
			e.printStackTrace();
			System.out.println("SQL Error: " + e.getMessage());
		}
		
		
		System.out.println("Comboboxes Built!");
		//####### END Build ComboBoxes ###########
	}
	
} //END Class MMCObject
