import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.sql.DriverManager;
import java.sql.SQLException;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;

/**************************************************************
 * TITLE:	Music Management Client 1.0
 * AUTHOR:	Andy Maskin
 * 			Networked Expression - Fall 2005 - O'Sullivan
 * 
 * MODULE: MusicMgmtClient
 * 
 * PURPOSE: This is the main module of the application and sets up a panel to enter connection
 * information. Once a connection is established, it builds an MMCObject and assembles all 
 * of the panels into tabbed panes.
 *
 **************************************************************/

public class MusicMgmtClient extends JFrame implements ActionListener{
	
	private static final long serialVersionUID = 1L;
	public static JPanel plConn = new JPanel();
	public static java.sql.Connection coDB;

    static JTextField tfServerName = new JTextField("",40);
    static JTextField tfDatabase = new JTextField("",40);
    static JTextField tfUserName = new JTextField("",40);
    static JTextField tfPassword = new JPasswordField("",40);
    static JLabel actionLabel = new JLabel("STATUS: OK");
    static JButton btnConnect = new JButton("Connect");
    static JButton btnDisconnect = new JButton("Disconnect");

	// CONSTRUCTOR
	public MusicMgmtClient() {
        vBuildConnPanel();
        this.getContentPane().add(plConn);
	    btnConnect.addActionListener(this);
	    btnDisconnect.addActionListener(this);
	    this.setTitle("Music Management Client");
    } //END constructor
	
	public static void main(String[] args) {
        System.out.println("Starting Music Management Client");
        JFrame.setDefaultLookAndFeelDecorated(true);

        //Create and set up the window.
        MusicMgmtClient frame = new MusicMgmtClient();
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.pack();
        frame.setResizable(false);
        frame.setSize(780,580);
        frame.validate();
        frame.setVisible(true);
        
    } //END main()
	
	public void vConnect(MMCObject _mmcConnectionInfo) {
		boolean bConnected = false;
		String sServer = tfServerName.getText();
		String sUsername = tfUserName.getText();
		String sPassword = tfPassword.getText();
		String sDatabase = tfDatabase.getText();
		
		//Connect to Database
		if (_mmcConnectionInfo == null) {
			// New Connection Scenario
			if (sServer.equals("") || sUsername.equals("") || sPassword.equals("") || sDatabase.equals("")) {
				vStatus("INCOMPLETE CONNECTION INFORMATION!");
			} else {
				vStatus("Connecting to " + sServer + " using the username " + sUsername + " ...");
				try {
				String sURL;
					try {
						Class.forName("com.mysql.jdbc.Driver");
					} catch (ClassNotFoundException e) {
						e.printStackTrace();
					}
				
				sURL = "jdbc:mysql://" + sServer + "/" + sDatabase + "?user=" + sUsername + "&password=" + sPassword;
				
				coDB = DriverManager.getConnection(sURL);
				
				vStatus("Connection Successful!");
				bConnected = true;
				}
				catch ( SQLException e) {
					e.printStackTrace();
					vStatus("Connection FAILED!");
				}  
			} 
			
		} else {
			// Refresh Scenario
		}
		
		//If connected, proceed 
		if (bConnected) {
			//SET UP BIG CONTAINER OBJECT
			MMCObject mmcToPass = new MMCObject(coDB, sUsername, sPassword, sServer, sDatabase);
			vBuildGUI(mmcToPass);
		} else {
			vStatus("-NOT CONNECTED-");
		} //end if connected
	
	} //END vConnect()
	
	public void vBuildGUI(MMCObject _mmcToPass) {
		// *** Called after connection established
		

		System.out.println("connection " + MMCObject.coDB.toString());
		//=== CREATE TABBED PANES
        ImageIcon icon = null;
		JTabbedPane tpMain = new JTabbedPane();
		JTabbedPane tpTag = new JTabbedPane();
		JTabbedPane tpPlaylist = new JTabbedPane();
		JTabbedPane tpGenre = new JTabbedPane();
		
		//=== CREATE PANELS
		ArtistPanel plArtist = new ArtistPanel();
		TitlePanel plTitle = new TitlePanel();
		AlbumPanel plAlbum = new AlbumPanel();
		CommentsPanel plComments = new CommentsPanel();
		OrigArtistPanel plOrigArtist = new OrigArtistPanel();
		AllTagsPanel plAllTags = new AllTagsPanel();
		PlaylistPanel plPlaylist = new PlaylistPanel();
		PlaylistEntryPanel plPlaylistEntry = new PlaylistEntryPanel();
		ValidGenrePanel plValidGenre = new ValidGenrePanel();
		GenreMatchPanel plGenreMatch = new GenreMatchPanel();
		GenreTranslatePanel plGenreTranslate = new GenreTranslatePanel();
		AnalysisPanel plAnalysis = new AnalysisPanel();
		TaggingPanel plTagging = new TaggingPanel();
		CommandsPanel plCommands = new CommandsPanel();
		
		//Populate tabbed panes
		tpTag.addTab("Artist", icon, plArtist,"Artist Tab");
		tpTag.setMnemonicAt(0, KeyEvent.VK_A);
		tpTag.addTab("Title", icon, plTitle,"Title Tab");
		tpTag.setMnemonicAt(1, KeyEvent.VK_T);
		tpTag.addTab("Album/Track", icon, plAlbum,"Album/Track Tab");
		tpTag.setMnemonicAt(2, KeyEvent.VK_L);
		tpTag.addTab("Comments", icon, plComments,"Comments Tab");
		tpTag.setMnemonicAt(3, KeyEvent.VK_C);
		tpTag.addTab("Original Artist", icon, plOrigArtist,"Original Artist Tab");
		tpTag.setMnemonicAt(4, KeyEvent.VK_O);
		tpTag.addTab("All Tag Info", icon, plAllTags,"All Tag Info Tab");
		tpTag.setMnemonicAt(5, KeyEvent.VK_G);
		
		tpPlaylist.addTab("Playlist", icon, plPlaylist,"Playlist Tab");
		tpPlaylist.setMnemonicAt(0, KeyEvent.VK_P);
		tpPlaylist.addTab("Playlist Entry", icon, plPlaylistEntry,"Playlist Entry Tab");
		tpPlaylist.setMnemonicAt(1, KeyEvent.VK_E);
		tpPlaylist.addTab("Tag", icon, plTagging,"Tagging Tab");
		tpPlaylist.setMnemonicAt(2, KeyEvent.VK_T);
		
		tpGenre.addTab("Valid Genre", icon, plValidGenre,"Valid Genres Tab");
		tpGenre.setMnemonicAt(0, KeyEvent.VK_V);
		tpGenre.addTab("Genre Match", icon, plGenreMatch,"Genre Match Tab");
		tpGenre.setMnemonicAt(1, KeyEvent.VK_M);
		tpGenre.addTab("Genre Translate", icon, plGenreTranslate,"Genre Translate Tab");
		tpGenre.setMnemonicAt(2, KeyEvent.VK_R);
		
		tpMain.addTab("Commands", icon, plCommands,"Commands Tab");
		tpMain.setMnemonicAt(0, KeyEvent.VK_C);
		tpMain.addTab("Tag Maintenance", icon, tpTag,"Tag Maintenance Tab");
		tpMain.setMnemonicAt(1, KeyEvent.VK_I);
		tpMain.addTab("Genre Management", icon, tpGenre,"Genre Management Tab");
		tpMain.setMnemonicAt(2, KeyEvent.VK_G);
		tpMain.addTab("Playlists & Tagging", icon, tpPlaylist,"Playlists And Tagging Tab");
		tpMain.setMnemonicAt(3, KeyEvent.VK_S);
		tpMain.addTab("Analysis", icon, plAnalysis,"Analysis Tab");
		tpMain.setMnemonicAt(4, KeyEvent.VK_Y);
		
		//=== REMOVE SERVER PANEL AND ADD MAIN TABBED PANE
		this.remove(plConn);
		this.getContentPane().add(tpMain);
		
	} //END vBuildGUI()
	
	public static void vBuildConnPanel() {
		JLabel lblServerName = new JLabel("Server Name");
		JLabel lblDatabase = new JLabel("Database");
	    JLabel lblUserName = new JLabel("User Name");
	    JLabel lblPassword = new JLabel("Password");
	    actionLabel.setForeground(Color.RED);
	    
	    
	    GridBagConstraints cLabels = new GridBagConstraints();
	    GridBagConstraints cFields = new GridBagConstraints();
	    GridBagConstraints cStatus = new GridBagConstraints();
	    
	    plConn.setLayout(new GridBagLayout());
	    
	    btnConnect.setName("Connect");
	    btnDisconnect.setName("Disconnect");

		cLabels.gridwidth = GridBagConstraints.RELATIVE; //next-to-last
		cLabels.fill = GridBagConstraints.NONE;      //reset to default
		cLabels.weightx = 0.0;                       //reset to default
		cLabels.weighty = 0.0;
		cLabels.ipadx = 5;
		cLabels.anchor = GridBagConstraints.FIRST_LINE_END;
		cLabels.insets = new Insets(10,30,3,3);
		cFields.gridwidth = GridBagConstraints.REMAINDER;     //end row
		cFields.fill = GridBagConstraints.NONE;
		cFields.weightx = 0.0;
		cFields.weighty = 0.0;
		cFields.ipadx = 5;
		cFields.anchor = GridBagConstraints.FIRST_LINE_START;
		cFields.insets = new Insets(10,3,3,30);
		cStatus.gridwidth = GridBagConstraints.REMAINDER; //last
	    cStatus.anchor = GridBagConstraints.LAST_LINE_START;
	    cStatus.weightx = 1.0;
	    cStatus.weighty = 1.0;
	    cStatus.insets = new Insets(30,10,10,30);
	    
	    btnConnect.setMnemonic(KeyEvent.VK_C);
	    btnConnect.setActionCommand("CONNECT_BUTTON");
	    btnDisconnect.setMnemonic(KeyEvent.VK_D);
	    btnDisconnect.setActionCommand("DISCONNECT_BUTTON");
	    
	    plConn.add(lblServerName, cLabels);
	    plConn.add(tfServerName, cFields);
	    plConn.add(lblDatabase, cLabels);
	    plConn.add(tfDatabase, cFields);
	    plConn.add(lblUserName, cLabels);
	    plConn.add(tfUserName, cFields);
	    plConn.add(lblPassword, cLabels);
	    plConn.add(tfPassword, cFields);
	    plConn.add(btnConnect, cLabels);
	    plConn.add(btnDisconnect, cFields);
	    plConn.add(actionLabel, cStatus);
		
	} //END vBuildConnPanel()
	
	public void vStatus (String sStatus) {
		actionLabel.setText(sStatus);
        System.out.println(sStatus);
        plConn.revalidate();
        plConn.repaint();
	} //END vStatus

	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getActionCommand().toString() == "CONNECT_BUTTON") {vConnect(null);}
		if (arg0.getActionCommand().toString() == "DISCONNECT_BUTTON") {vDisconnect();}
	} //end actionPerformed
	
	public void vDisconnect(){
		vStatus("Disconnecting!");
		try { coDB.close(); }
		catch (Exception e ) {e.printStackTrace();}
	} //end Disconnect
	

} //END CLASS MusicMgmtClient
