import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.Connection;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

/**
 * Music Management Client 1.0
 * 
 * MODULE: NewPlaylistPanel
 * 
 * AUTHOR: Andy Maskin
 * 
 * PURPOSE: This panel affords the user an opportunity to enter a new playlist record
 *
 */

public class NewPlaylistPanel extends JPanel implements ActionListener {
	
	private static final long serialVersionUID = 1L;
	public static Connection coDB;
	
	//	sub-panel for adding new records
	JLabel lblNewTitle = new JLabel("Title: ");
	JLabel lblNewYear = new JLabel("Year: ");
	JLabel lblNewArtist = new JLabel("Artist: ");
	JLabel lblNewType = new JLabel("Type: ");
	TitledBorder ttNewRecord;
	AComboBox acbNewArtist = new AComboBox ();
	JTextField tfNewYear = new JTextField("",40);
	JTextField tfNewTitle = new JTextField("",150);
	JComboBox cmbNewType = new JComboBox();
    JButton btnAdd = new JButton("Add");
	JLabel lblNewMessage = new JLabel("|");
	
	GridBagConstraints cNewStatus = new GridBagConstraints();
    GridBagConstraints cNewLabels = new GridBagConstraints();
    GridBagConstraints cNewFields = new GridBagConstraints();
    
    String sNewType;
    String sNewArtist;
    String sNewTitle;
    String sNewYear;
	
    {
    		//PASSED-IN VALUES
    		acbNewArtist.addItem("");
    		for (int i=0; i < MMCObject.vAFFN.size(); i++){acbNewArtist.addItem(MMCObject.vAFFN.elementAt(i));}
    		cNewStatus = MMCObject.cStatus;
    		cNewLabels = MMCObject.cLabels;
    		cNewFields = MMCObject.cFields;
    		coDB = MMCObject.coDB;
    		
    		//cmbPlaylistType
    		cmbNewType.addItem("ALB");
    		cmbNewType.addItem("MIX");
    		
    		//set up new record panel
    	    lblNewMessage.setForeground(Color.RED);
    	    lblNewMessage.setAlignmentX((float) 0.5);
    	    this.setLayout(new GridBagLayout());
    	    ttNewRecord = BorderFactory.createTitledBorder(" - New Playlist Details - ");
    	    ttNewRecord.setTitleFont(new Font("Arial", Font.PLAIN, 14));
    	    ttNewRecord.setTitleJustification(TitledBorder.CENTER);
    	    tfNewTitle.setMinimumSize(new Dimension(500,20));
    	    tfNewYear.setMinimumSize(new Dimension(100,20));
    	    btnAdd.addActionListener(this);
    	    this.setBorder(ttNewRecord);
    	    this.setBackground(Color.WHITE);
    	    this.add(lblNewType,cNewLabels);
    	    this.add(cmbNewType,cNewFields);
    	    this.add(lblNewArtist,cNewLabels);
    	    this.add(acbNewArtist,cNewFields);
    	    this.add(lblNewTitle,cNewLabels);
    	    this.add(tfNewTitle,cNewFields);
    	    this.add(lblNewYear,cNewLabels);
    	    this.add(tfNewYear,cNewFields);
    	    this.add(btnAdd,cNewStatus);
    	    this.add(lblNewMessage,cNewStatus);
    	    this.setMinimumSize(new Dimension(700,300));
    	    this.setMaximumSize(new Dimension(700,300));
    	    this.setPreferredSize(new Dimension(700,300));
    	    this.setSize(new Dimension(700,300));

    }


	public void actionPerformed(ActionEvent arg0) {
		//Add Record Button
		if (arg0.getActionCommand().equals("Add")) {
			sNewType = cmbNewType.getSelectedItem().toString();
		    sNewArtist = acbNewArtist.getSelectedItem().toString();
		    sNewTitle= tfNewTitle.getText();
		    sNewYear= tfNewYear.getText();
		    if (bValidated()) {vAddRecord();}
		} //end add record call
	}
	
	public boolean bValidated(){
		boolean bResult = true;
		
		if (sNewTitle.equals("") && bResult) {
			vStatus("Error! Playlists must all have a title!");
			bResult = false;
		}
		
		if (bResult && sNewType.equals("ALB") && (sNewArtist.equals("") || sNewYear.equals(""))) {
			vStatus("Error! Playlists of type ALB need both an Artist and a Year!");
			bResult = false;
		}
		
		//Validate Year
		if (sNewYear.equals("") == false) {
			try {
				if (Integer.parseInt(sNewYear) < 1900 && Integer.parseInt(sNewYear) > 2010) {
					vStatus("Error! Invalid Year!");
					bResult = false;
				}
			} catch (Exception e) {
				vStatus("Error! Invalid Year!");
				bResult = false;
			}
		} //end year test
		
		return bResult;
	}
	
	public void vStatus(String sMessage){
		lblNewMessage.setText(sMessage);
		System.out.println(sMessage);
	}
	
	public void vAddRecord(){
		String sSQL = "";
		boolean bResult = true;
		
		vStatus("|");
		try {
			sSQL = "INSERT INTO Playlist (Type, ArtistFromFilename, Title, Year, User) VALUES (";
			sSQL = sSQL + "\"" + sNewType + "\", \"" + sNewArtist + "\", \"" + sNewTitle + "\", ";
			sSQL = sSQL + "\"" + sNewYear + "\", \"" + MMCObject.sUsername + "\")";
			System.out.println(sSQL);
			bResult = MMCObject.executeSQL(sSQL);
		} catch (Exception e){
			System.out.println(e.toString());
			vStatus("ERROR INSERTING RECORD! Check for existing Title!");
		}
		
		if (bResult == false) {
			vStatus("ERROR INSERTING RECORD! Check for existing Title!");
		} else {
			//Clear fields
			acbNewArtist.setSelectedIndex(0);
		    tfNewTitle.setText("");
		    tfNewYear.setText("");
		}
	} //end Add Record method
	
	

} //END CLASS
