import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 * Music Management Client 1.0
 * 
 * MODULE: PlaylistEntryPanel
 * 
 * AUTHOR: Andy Maskin
 * 
 * PURPOSE: This panel allows users to add or remove files from a given playlist
 *
 */

public class PlaylistEntryPanel extends JPanel implements TableModelListener, ActionListener {

	private static final long serialVersionUID = 1L;
	public static Connection coDB;
	
	// GRAPHIC ELEMENTS
	public static JLabel lblName = new JLabel("Playlist: ");
	public static AComboBox acbPlaylist = new AComboBox ();
	public static JLabel lblError = new JLabel("|");
    public static JButton btnDelete = new JButton("Delete");
    public static AndyGrid tbResults = new AndyGrid(0,0);
    public static JScrollPane spResults;
    
    //sub-panel for adding new records
    public static JPanel plNewRecord = new JPanel();
	public static JLabel lblNewFile = new JLabel("File: ");
	public static JLabel lblNewTrack = new JLabel("Track: ");
	public static TitledBorder ttNewRecord;
	public static AComboBox acbFile = new AComboBox ();
	public static JTextField tfTrack = new JTextField("",40);
    public static JButton btnAdd = new JButton("Add");
    
    GridBagConstraints cStatus = new GridBagConstraints();
    GridBagConstraints cLabels = new GridBagConstraints();
    GridBagConstraints cFields = new GridBagConstraints();
    
    {
    		//PASSED-IN VALUES
		coDB = MMCObject.coDB;
		acbPlaylist = MMCObject.acbPlaylist;
		acbFile.addItem("");
		for (int i=0; i < MMCObject.vFilename.size(); i++){acbFile.addItem(MMCObject.vFilename.elementAt(i));}
		cStatus = MMCObject.cStatus;
		cLabels = MMCObject.cLabels;
		cFields = MMCObject.cFields;
		
    		// LAYOUT STUFF
    		spResults = new JScrollPane(tbResults);
        tbResults.setPreferredScrollableViewportSize(new Dimension(700, 250));
        spResults.setMinimumSize(new Dimension(700, 250));
        
        acbPlaylist.setEditable(true);
        acbPlaylist.addActionListener(this);
        
        acbFile.setEditable(true);
        acbFile.addActionListener(this);
	    
	    //add/delete buttons
		btnAdd.addActionListener(this);
		btnDelete.addActionListener(this);
    	   
    	    //set up new record panel
    	    plNewRecord.setLayout(new GridBagLayout());
    	    ttNewRecord = BorderFactory.createTitledBorder("New Record");
    	    plNewRecord.setBorder(ttNewRecord);
    	    plNewRecord.add(lblNewFile,cLabels);
    	    plNewRecord.add(acbFile,cFields);
    	    plNewRecord.add(lblNewTrack,cLabels);
    	    tfTrack.setMinimumSize(new Dimension(100,20));
    	    plNewRecord.add(tfTrack,cFields);
    	    plNewRecord.add(btnAdd,cStatus);
    	    
    	    lblError.setForeground(Color.RED);
    	    lblError.setAlignmentX((float) 0.5);
    	    
    		this.setLayout(new GridBagLayout());
    		this.add(lblName,cLabels);
    	    this.add(acbPlaylist,cFields);
    		this.add(plNewRecord,cStatus);
    	    this.add(lblError,cStatus);
    		this.add(btnDelete,cStatus);
    	    this.add(spResults,cStatus);
    	
    } //end construction


    public void actionPerformed(ActionEvent arg0) {
		System.out.println(arg0.toString());
		//System.out.println(arg0.getModifiers());
		ComboItem ciSelected = null;
		//New Playlist Chosen
		if (arg0.getActionCommand().equals("comboBoxEdited") ||
				(arg0.getActionCommand().equals("comboBoxChanged") && arg0.getModifiers()==16)) {
			
			//Jan 2006 added these two lines to deal with "return" key throwing errors 
			acbPlaylist.repaint();
			acbPlaylist.setSelectedIndex(acbPlaylist.getAnIndex(acbPlaylist.getSelectedItem().toString()));
			
			ciSelected = (ComboItem) acbPlaylist.getSelectedItem();
	    	    Statement stSQL;
			ResultSet rsResults;
			String sSQL;
			
			sSQL = "SELECT pe.row_id as X, pe.filename as PL_File, pe.track as PL_Track from PlaylistEntry pe ";
			sSQL = sSQL + "WHERE Playlist_ID=" + ciSelected.getID();
			
			try {
				Vector columnNames = new Vector();
				Vector data = new Vector();
				stSQL = coDB.createStatement();
				rsResults = stSQL.executeQuery(sSQL);
	
				ResultSetMetaData mdResults = rsResults.getMetaData();
				int iColumnCount = mdResults.getColumnCount();
				
				if (rsResults.first()) {System.out.println("Records Returned");}
				
				//get ColumnNames
				for(int i=1; i <= iColumnCount ;i++){
					columnNames.addElement( mdResults.getColumnName(i) );
				}
				
				//get data
				boolean bContinue = rsResults.first();
				while (bContinue)
	            {
	                Vector row = new Vector(iColumnCount);
	                for (int i = 1; i <= iColumnCount; i++)
	                {
	                    row.addElement( rsResults.getObject(i) );
	                }
	                data.addElement( row );
	                bContinue = rsResults.next();
	            }
				
		        rsResults.close();
	            stSQL.close();
	            AndyGrid tbTemp = new AndyGrid(data, columnNames); 
	            this.setResultsTable(tbTemp);
				
			} // end try
			catch (SQLException e) {
				e.printStackTrace();
				System.out.println("SQL Error: " + e.getMessage());
				Vector columnNames = new Vector();
				Vector data = new Vector();
				Vector row = new Vector(1);
				columnNames.addElement("X");
				columnNames.addElement("SQL ERROR");
				row.addElement("!");
				row.addElement(e.getMessage());
				data.addElement(row);
				AndyGrid tbTemp = new AndyGrid(data, columnNames); 
	            	this.setResultsTable(tbTemp);
			} //end exception
		} // end if combo box edited
		
		//Add Button
		if (arg0.getActionCommand().equals("Add")) {
			System.out.println("attempting to add record");
			vAddRecord();
		} //end add record call
		
		//Delete Button
		if (arg0.getActionCommand().equals("Delete")) {
			System.out.println("attempting to delete record");
			tbResults.vDeleteRecord();
			refreshGrid();
		} //end add record call
	}

    private void vAddRecord() {
    		boolean bOK = true;

    		//Validate required information
    		if (acbPlaylist.getSelectedItem().toString().equals("")) {
    			System.out.println("Error! Playlist not chosen!");
    			lblError.setText("Error! Playlist not chosen!");
    			bOK = false;
    		}
    		if (acbFile.getSelectedItem().toString().equals("")) {
    			System.out.println("Error! File not chosen!");
    			lblError.setText("Error! File not chosen!");
    			bOK = false;
    		}
    		if (tfTrack.getText().equals("")) {
    			System.out.println("Error! Track not chosen!");
    			lblError.setText("Error! Track not chosen!");
    			bOK = false;
    		}
    		
    		// Check to make sure Track # is valid
    		if (bOK == true) {
    			try {
    				int iTest;
    				iTest = Integer.parseInt(tfTrack.getText().toString());
    				if (iTest == 9999) {System.out.println(iTest);}
    			} catch (Exception e){
        			System.out.println("Error! Track must be an Integer!");
        			lblError.setText("Error! Track must be an Integer");
        			bOK = false;
    			}
    		} //end integer check
    		
    		//check to make sure its not a duplicate track #
    		if (bOK == true && tbResults.bFindMatchInColumn(2,tfTrack.getText().toString()) == true)
		{
    			System.out.println("Error! Track already exists in this playlist!");
    			lblError.setText("Error! Track already exists in this playlist!");
    			bOK = false;
		}
    		
    		if (bOK == true) {
    			lblError.setText("|");
    			try {
    				ComboItem ciSelected = null;
    				ciSelected = (ComboItem) acbPlaylist.getSelectedItem();
    				String sSQL = "";
    				sSQL = "INSERT INTO PlaylistEntry (Playlist_ID, Filename, Track) VALUES (";
    				sSQL = sSQL + ciSelected.getID() + ", \"" + acbFile.getSelectedItem().toString();
    				sSQL = sSQL + "\", " + tfTrack.getText().toString() + ")";
        			System.out.println(sSQL);
        			tbResults.executeSQL(sSQL,0,0);
        			refreshGrid();
    			} catch (Exception e){
        			System.out.println(e.toString());
    			}
    		} //end if everything is OK - RUN INSERT
	} //end addRecord method
    

    public void refreshGrid() {
    		Object oTemp;
    		oTemp = acbPlaylist.getSelectedItem();
    		acbPlaylist.setSelectedItem(oTemp);
    } //end refresh grid
    
	private void setResultsTable(AndyGrid _tbResults) {
		tbResults = _tbResults;
		System.out.println((tbResults.getColumnCount()) + " columns");
		System.out.println((tbResults.getRowCount()) + " rows");
	    this.remove(spResults);
	    tbResults.formatGrid();
	    tbResults.getModel().addTableModelListener(this);
		spResults = new JScrollPane(tbResults);
	    spResults.setMinimumSize(new Dimension(700,250));
	    spResults.setHorizontalScrollBarPolicy(32);
	    spResults.setVerticalScrollBarPolicy(22);
	    this.add(spResults,cStatus);
	    this.repaint();
	    this.revalidate();
	    spResults.requestFocus();
	    tbResults.requestFocus();
	}

	public void tableChanged(TableModelEvent e) {
		int row = e.getFirstRow();
        	int column = e.getColumn();
        	if ( column != 0) {
	        	TableModel model = (TableModel)e.getSource();
	        tbResults.captureUpdate(row, column, (String) model.getValueAt(row, column));
        } //cannot edit key column
    }

	
	
}  // END CLASS
