import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 * Music Management Client 1.0
 * 
 * MODULE: PlaylistPanel
 * 
 * AUTHOR: Andy Maskin
 * 
 * PURPOSE: This panel is where users can manipulate the title, artist and year information
 * of existing playlists, or delete them
 *
 */

public class PlaylistPanel extends JPanel implements TableModelListener, ActionListener {

	private static final long serialVersionUID = 1L;
	public static Connection coDB;
	
	// GRAPHIC ELEMENTS
	public static JLabel lblName = new JLabel("Artist: ");
	public static JLabel lblType = new JLabel("Type: ");
	public static JLabel lblUser = new JLabel("User: ");
	public static AComboBox acbPick = new AComboBox ();
	public static JComboBox cmbType = new JComboBox ();
	public static JComboBox cmbUsers = new JComboBox ();
    public static JButton btnDelete = new JButton("Delete");
    public static JButton btnNewRecord = new JButton("New Record");
    public static AndyGrid tbResults = new AndyGrid(0,0);
    public static JScrollPane spResults;
    
    public static NewPlaylistPanel plNewRecord = null;

    String sMode = "Normal";
    
    GridBagConstraints cStatus = new GridBagConstraints();
    GridBagConstraints cLabels = new GridBagConstraints();
    GridBagConstraints cFields = new GridBagConstraints();
	
    {
    		//PASSED-IN VALUES
    		coDB = MMCObject.coDB;
    		acbPick.addItem("");
    		for (int i=0; i < MMCObject.vAFFN.size(); i++){acbPick.addItem(MMCObject.vAFFN.elementAt(i));}
    		cmbType = MMCObject.cmbPlaylistType;
    		cmbUsers = MMCObject.cmbUsers;
    		cStatus = MMCObject.cStatus;
    		cLabels = MMCObject.cLabels;
    		cFields = MMCObject.cFields;
    	
    		//LAYOUT STUFF
		spResults = new JScrollPane(tbResults);
		tbResults.setPreferredScrollableViewportSize(new Dimension(700,300));
		spResults.setMinimumSize(new Dimension(700,300));

		acbPick.setEditable(true);
		acbPick.addActionListener(this);
		cmbType.setEditable(false);
		cmbType.addActionListener(this);
		cmbUsers.setEditable(false);
		cmbUsers.addActionListener(this);
		btnDelete.addActionListener(this);
		btnNewRecord.addActionListener(this);
    	
		this.setLayout(new GridBagLayout());
		this.add(lblType,cLabels);
		this.add(cmbType,cFields);
		this.add(lblName,cLabels);
	    	this.add(acbPick,cFields);
	    	this.add(lblUser,cLabels);
		this.add(cmbUsers,cFields);
		this.add(btnDelete,cStatus);
		this.add(btnNewRecord,cStatus);
	    	this.add(spResults,cStatus);
	    
    }
    
    public void actionPerformed(ActionEvent arg0) {
		System.out.println(arg0.toString());
		System.out.println(arg0.getModifiers());
		if (arg0.getActionCommand().equals("comboBoxEdited") ||
				(arg0.getActionCommand().equals("comboBoxChanged") && arg0.getModifiers()==16)) {
	    	    Statement stSQL;
			ResultSet rsResults;
			String sSQL;
			String sChosenType = (String) cmbType.getSelectedItem();
			String sChosenUser = (String) cmbUsers.getSelectedItem();
			String sChosenArtist = (String) acbPick.getSelectedItem();
			
			sSQL = "SELECT ROW_ID, ArtistFromFilename as PL_Artist, Title as PL_Title, Year as PL_Year, Type as PL_Type, User from Playlist WHERE ROW_ID is not null ";
			if (sChosenType.equals("Any") == false) {sSQL = sSQL + " AND Type =\"" + sChosenType + "\"";}
			if (sChosenUser.equals("Any") == false) {sSQL = sSQL + " AND User =\"" + sChosenUser + "\"";}
			if (sChosenArtist.equals("") == false) {sSQL = sSQL + " AND ArtistFromFilename =\"" + sChosenArtist + "\"";}
			
			sSQL = sSQL + " ORDER By ArtistFromFilename, Title";
			

			System.out.println(sSQL);
			
			try {
				Vector columnNames = new Vector();
				Vector data = new Vector();
				stSQL = coDB.createStatement();
				rsResults = stSQL.executeQuery(sSQL);
	
				ResultSetMetaData mdResults = rsResults.getMetaData();
				int iColumnCount = mdResults.getColumnCount();
				
				if (rsResults.first()) {System.out.println("Records Returned");}
				
				//get ColumnNames
				for(int i=1; i <= iColumnCount ;i++){
					columnNames.addElement( mdResults.getColumnName(i) );
				}
				
				//get data
				boolean bContinue = rsResults.first();
				while (bContinue)
	            {
	                Vector row = new Vector(iColumnCount);
	                for (int i = 1; i <= iColumnCount; i++)
	                {
	                    row.addElement( rsResults.getObject(i) );
	                }
	                data.addElement( row );
	                bContinue = rsResults.next();
	            }
				
		        rsResults.close();
	            stSQL.close();
	            AndyGrid tbTemp = new AndyGrid(data, columnNames); 
	            setResultsTable(tbTemp);
				
			} // end try
			catch (SQLException e) {
				e.printStackTrace();
				System.out.println("SQL Error: " + e.getMessage());
				Vector columnNames = new Vector();
				Vector data = new Vector();
				Vector row = new Vector(1);
				columnNames.addElement("X");
				columnNames.addElement("SQL ERROR");
				row.addElement("!");
				row.addElement(e.getMessage());
				data.addElement(row);
				AndyGrid tbTemp = new AndyGrid(data, columnNames); 
	            	this.setResultsTable(tbTemp);
			} //end exception
		} // end if edited
		
		//Delete Button
		if (arg0.getActionCommand().equals("Delete") && sMode.equals("Normal")) {
			tbResults.vDeleteRecord();
			refreshGrid();
		} //end delete record call
		
		//New Record Button
		if (arg0.getActionCommand().equals("New Record")) {
			if (sMode.equals("Normal")) {showNewRecord();}
		} //end delete record call
	}
    
    public void refreshGrid() {
		Object oTemp;
		oTemp = cmbType.getSelectedItem();
		cmbType.setSelectedItem(oTemp);
    } //end refresh grid
    
    public void showNewRecord() { 
    		sMode = "NewRecord";
		System.out.println("NewRecord Mode");
		this.remove(spResults);
    		plNewRecord = new NewPlaylistPanel();
    		plNewRecord.setPreferredSize(new Dimension (700,300));
    		plNewRecord.setMaximumSize(new Dimension (700,300));
    		plNewRecord.setSize(new Dimension (700,300));
    		this.add(plNewRecord,cStatus);
    	    this.repaint();
    	    this.revalidate();
    }
    
    public void hideNewRecord() {
		sMode = "Normal";
		System.out.println("Normal Mode");
		this.remove(plNewRecord);
		this.add(spResults,cStatus);
		coDB = MMCObject.coDB;		//reclaim connection from newRecord panel
    }

    private void setResultsTable(AndyGrid _tbResults) {
    		if (sMode.equals("NewRecord")) {hideNewRecord();}	//hide new record panel if showing
		tbResults = _tbResults;
		System.out.println((tbResults.getColumnCount()) + " columns");
		System.out.println((tbResults.getRowCount()) + " rows");
	    this.remove(spResults);
	    tbResults.formatGrid();
	    tbResults.getModel().addTableModelListener(this);
	    spResults.add(tbResults);
		spResults = new JScrollPane(tbResults);
	    spResults.setMinimumSize(new Dimension(700,300));
		spResults.setSize(new Dimension(700,300));
		spResults.setPreferredSize(new Dimension(700,300));
	    //spResults.setHorizontalScrollBarPolicy(32);
	    //spResults.setVerticalScrollBarPolicy(22);
	    this.add(spResults,cStatus);
	    this.repaint();
	    this.revalidate();
	    spResults.requestFocus();
	    tbResults.requestFocus();
	}

	public void tableChanged(TableModelEvent e) {
		int row = e.getFirstRow();
        	int column = e.getColumn();
        	if ( column != 0) {
	        	TableModel model = (TableModel)e.getSource();
	        tbResults.captureUpdate(row, column, (String) model.getValueAt(row, column));
        } //cannot edit key column
    }
    
} //END CLASS
