import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 * Music Management Client 1.0
 * 
 * MODULE: ValidGenrePanel
 * 
 * AUTHOR: Andy Maskin
 * 
 * PURPOSE: This panel is where users maintain the list of Genres considered to be valid
 *
 */

public class ValidGenrePanel extends JPanel implements TableModelListener, ActionListener {

	private static final long serialVersionUID = 1L;
	public static Connection coDB;
	
	// GRAPHIC ELEMENTS
	public static JLabel lblGenre = new JLabel("New Genre: ");
	public static JLabel lblMessage = new JLabel("|");
	public static JTextField tfNewGenre = new JTextField();
    public static JButton btnDelete = new JButton("Delete");
    public static JButton btnAdd = new JButton("Add");
    public static AndyGrid tbResults = new AndyGrid(0,0);
    public static JScrollPane spResults;
    
    GridBagConstraints cStatus = new GridBagConstraints();
    GridBagConstraints cLabels = new GridBagConstraints();
    GridBagConstraints cFields = new GridBagConstraints();
	
    {
    		//PASSED-IN VALUES
    		coDB = MMCObject.coDB;
    		cStatus = MMCObject.cStatus;
    		cLabels = MMCObject.cLabels;
    		cFields = MMCObject.cFields;
    	
    		//LAYOUT STUFF
		spResults = new JScrollPane(tbResults);
		tbResults.setPreferredScrollableViewportSize(new Dimension(700,300));
		spResults.setMinimumSize(new Dimension(700,300));
		

	    tfNewGenre.setMinimumSize(new Dimension(300,20));
	    tfNewGenre.setPreferredSize(new Dimension(300,20));
	    lblMessage.setForeground(Color.RED);
	    lblMessage.setAlignmentX((float) 0.5);

		btnDelete.addActionListener(this);
		btnAdd.addActionListener(this);
    	
		this.setLayout(new GridBagLayout());
		this.add(lblGenre,cLabels);
		this.add(tfNewGenre,cFields);
		this.add(btnAdd,cLabels);
	    	this.add(btnDelete,cFields);
		this.add(lblMessage,cStatus);
	    	this.add(spResults,cStatus);
	    	
	    	refreshGrid();
    }
    
    public void actionPerformed(ActionEvent arg0) {
		
		//New Record Button
		if (arg0.getActionCommand().equals("Add")) {
			if (tfNewGenre.getText().equals("")) {
				vStatus("Genre field is blank!");
			} else {
				vAddRecord();
			}
		} //end new record call
		
		//Delete Button
		if (arg0.getActionCommand().equals("Delete")) {
			tbResults.vDeleteRecord();
			refreshGrid();
		} //end delete record call
	}
    
    public void vAddRecord(){
		String sSQL = "";
		boolean bResult = true;
		
		vStatus("|");
		try {
			sSQL = "INSERT INTO Genre (Genre, Active) VALUES(";
			sSQL = sSQL + "\"" + tfNewGenre.getText() + "\", \"Y\")";
			System.out.println(sSQL);
			bResult = MMCObject.executeSQL(sSQL);
			sSQL = "DELETE FROM GenreTranslate WHERE OldGenre = \"" + tfNewGenre.getText() + "\"";
			System.out.println(sSQL);
			bResult = MMCObject.executeSQL(sSQL);
			refreshGrid();
		} catch (Exception e){
			System.out.println(e.toString());
			vStatus("ERROR INSERTING RECORD! Check for existing Genre!");
		}
		
		if (bResult == false) {
			vStatus("ERROR INSERTING RECORD! Check for existing Genre!");
		} else {
			//Clear field
		    tfNewGenre.setText("");
		}
	} //end Add Record method
    
    public void refreshGrid() {
    	Statement stSQL;
		ResultSet rsResults;
		String sSQL;
		
		sSQL = "SELECT Genre as TheKey, Genre from Genre WHERE Active = 'Y'";
		sSQL = sSQL + " ORDER By Genre";
		
		try {
			Vector columnNames = new Vector();
			Vector data = new Vector();
			stSQL = coDB.createStatement();
			rsResults = stSQL.executeQuery(sSQL);

			ResultSetMetaData mdResults = rsResults.getMetaData();
			int iColumnCount = mdResults.getColumnCount();
			
			if (rsResults.first()) {System.out.println("Records Returned");}
			
			//get ColumnNames
			for(int i=1; i <= iColumnCount ;i++){
				columnNames.addElement( mdResults.getColumnName(i) );
			}
			
			//get data
			boolean bContinue = rsResults.first();
			while (bContinue)
            {
                Vector row = new Vector(iColumnCount);
                for (int i = 1; i <= iColumnCount; i++)
                {
                    row.addElement( rsResults.getObject(i) );
                }
                data.addElement( row );
                bContinue = rsResults.next();
            }
			
	        rsResults.close();
            stSQL.close();
            AndyGrid tbTemp = new AndyGrid(data, columnNames); 
            setResultsTable(tbTemp);
			
		} // end try
		catch (SQLException e) {
			e.printStackTrace();
			System.out.println("SQL Error: " + e.getMessage());
			Vector columnNames = new Vector();
			Vector data = new Vector();
			Vector row = new Vector(1);
			columnNames.addElement("X");
			columnNames.addElement("SQL ERROR");
			row.addElement("!");
			row.addElement(e.getMessage());
			data.addElement(row);
			AndyGrid tbTemp = new AndyGrid(data, columnNames); 
            	this.setResultsTable(tbTemp);
		} //end exception
    } //end refresh grid
    
    public void vStatus(String sMessage){
		lblMessage.setText("  " + sMessage);
		System.out.println(sMessage);
	}

    private void setResultsTable(AndyGrid _tbResults) {
		tbResults = _tbResults;
		System.out.println((tbResults.getColumnCount()) + " columns");
		System.out.println((tbResults.getRowCount()) + " rows");
	    this.remove(spResults);
	    tbResults.formatGrid();
	    tbResults.getModel().addTableModelListener(this);
	    spResults.add(tbResults);
		spResults = new JScrollPane(tbResults);
	    spResults.setMinimumSize(new Dimension(700,300));
		spResults.setSize(new Dimension(700,300));
		spResults.setPreferredSize(new Dimension(700,300));
	    //spResults.setHorizontalScrollBarPolicy(32);
	    //spResults.setVerticalScrollBarPolicy(22);
	    this.add(spResults,cStatus);
	    this.repaint();
	    this.revalidate();
	    spResults.requestFocus();
	    tbResults.requestFocus();
	}

	public void tableChanged(TableModelEvent e) {
		int row = e.getFirstRow();
        	int column = e.getColumn();
        	if ( column != 0) {
	        	TableModel model = (TableModel)e.getSource();
	        tbResults.captureUpdate(row, column, (String) model.getValueAt(row, column));
        } //cannot edit key column
    }
    
} //END CLASS
